/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.audit.destination;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.ranger.audit.server.AuditConfig;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Properties;

@Component
public class AuditDestinationMgr {

    private static final Logger LOG = LoggerFactory.getLogger(AuditDestinationMgr.class);

    AuditDestination auditDestination;

    @PostConstruct
    public void configure() throws Exception {
        init();
    }

    public void init(){
        LOG.info("==> AuditDestinationMgr.init()");

        AuditConfig auditConfig = AuditConfig.getInstance();
        Properties properties = auditConfig.getProperties();
        if (properties != null) {
            auditDestination = new OpenSearchAuditDestination();
            auditDestination.init(properties);
        }

        LOG.info("<== AuditDestinationMgr.init()");
    }

    public AuditDestination getAuditDestination() {
        if (auditDestination == null) {
            init();
        }
        return auditDestination;
    }
}
