/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.audit.rest;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import org.apache.http.HttpStatus;
import org.apache.ranger.audit.destination.AuditDestination;
import org.apache.ranger.audit.destination.AuditDestinationMgr;
import org.apache.ranger.audit.model.AuthzAuditEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.ws.rs.*;
import javax.ws.rs.core.Response;

@Path("/audit")
@Component
@Scope("request")
public class AuditREST {
    private static final Logger LOG = LoggerFactory.getLogger(AuditREST.class);
    private Gson gson =  new GsonBuilder().setDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").create();

    @Autowired
    AuditDestinationMgr auditDestinationMgr;

    @GET
    @Path("/{param}")
    public Response getMsg(@PathParam("param") String msg) {
        String output = "Message received : " + msg;
        LOG.info(output);
        return Response.status(200).entity(output).build();
    }

    @POST
    @Path("/post")
    @Consumes({ "application/json", "application/xml" })
    @Produces({ "application/json", "application/xml" })
    public Response postAudit(String data) {
        Response ret = null;
        AuthzAuditEvent authzAuditEvent = gson.fromJson(data, AuthzAuditEvent.class);
        if (LOG.isDebugEnabled()) {
            LOG.info("==> AuditREST. postAudit() : AuthzAuditEvent: " +  authzAuditEvent);
        }
        try {
            if (auditDestinationMgr != null) {
                AuditDestination opensearchAuditDestination = auditDestinationMgr.getAuditDestination();
                boolean result = opensearchAuditDestination.log(authzAuditEvent);
                ret = (result) ?  Response.status(HttpStatus.SC_OK).entity(data).build() : Response.status(HttpStatus.SC_BAD_REQUEST).entity(data).build();
            }
        } catch (Exception e) {
            LOG.error("Error Sending Audit to OpenSearch...", e);
        }
        return ret;
        }
}
