/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.audit.server;

import org.apache.log4j.Logger;
import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
import java.util.Properties;

public class AuditConfig extends RangerConfiguration {

    private static final Logger LOG = Logger.getLogger(AuditConfig.class);
    public static final String CONFIG_FILE_PATH = "conf/audit-server-site.xml";
    private static volatile AuditConfig sInstance = null;

    public static AuditConfig getInstance() {
        AuditConfig ret = AuditConfig.sInstance;

        if (ret == null) {
            synchronized (AuditConfig.class) {
                ret = AuditConfig.sInstance;

                if (ret == null) {
                    ret = AuditConfig.sInstance = new AuditConfig();
                }
            }
        }

        return ret;
    }

    private AuditConfig() {
        super();
        addAuditResources();
    }

    public Properties getProperties() {
        return this.getProps();
    }

    private boolean addAuditResources() {
        if (LOG.isDebugEnabled()) {
            LOG.debug("==> addAuditResources()");
        }

        boolean ret = true;

        if (!addResourceIfReadable(CONFIG_FILE_PATH)) {
            LOG.error("Could not add " + CONFIG_FILE_PATH + " to AuditConfig.");
            ret = false;
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("<== addAuditResources(), result=" + ret);
        }

        return ret;
    }
}
