#!/bin/bash

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#
# Downloads HDFS/Hive/HBase/Kafka/.. archives to a local cache directory.
# The downloaded archives will be used while building docker images that
# run these services
#


#
# source .env file to get versions to download
#
source .env
ranger_url="${CDH_TAR_URL}/tars/ranger"

downloadIfNotPresent() {
  local fileName=$1
  local urlBase=$2

  if [ ! -f "downloads/${fileName}" ]
  then
    echo "downloading ${urlBase}/${fileName}.."

    curl -L ${urlBase}/${fileName} --output downloads/${fileName}
  else
    echo "file already in cache: ${fileName}"
  fi
}

downloadRangerArtifacts() {
  local fileName=$1
  local urlBase=$2

  if [ ! -f "dist/${fileName}" ]
  then
    echo "downloading ${urlBase}/${fileName} .."

    curl -L ${urlBase}/${fileName} --output dist/${fileName}
  else
    echo "file already in cache: ${fileName}"
  fi
}

install_dependencies(){
  downloadIfNotPresent hadoop-${HADOOP_VERSION}.tar.gz        "${CDH_TAR_URL}"/tars/hadoop
  downloadIfNotPresent hbase-${HBASE_VERSION}-bin.tar.gz      "${CDH_TAR_URL}"/tars/hbase
  downloadIfNotPresent kafka_2.12-${KAFKA_VERSION}.tgz        "${CDH_TAR_URL}"/tars/kafka
  downloadIfNotPresent apache-hive-${HIVE_VERSION}-bin.tar.gz "${CDH_TAR_URL}"/tars/hive
  downloadIfNotPresent hadoop-${HIVE_HADOOP_VERSION}.tar.gz   "${CDH_TAR_URL}"/tars/hadoop
  downloadIfNotPresent postgresql-42.2.16.jre7.jar            https://search.maven.org/remotecontent?filepath=org/postgresql/postgresql/42.2.16.jre7
  downloadIfNotPresent mysql-connector-java-8.0.28.jar        https://search.maven.org/remotecontent?filepath=mysql/mysql-connector-java/8.0.28
  downloadIfNotPresent log4jdbc-1.2.jar                       https://repo1.maven.org/maven2/com/googlecode/log4jdbc/log4jdbc/1.2
}

install_ranger_artifacts(){
  downloadRangerArtifacts ranger-${RANGER_VERSION}-admin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-hdfs-plugin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-yarn-plugin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-hive-plugin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-hbase-plugin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-kafka-plugin.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-rms.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-raz.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-usersync.tar.gz ${ranger_url}
  downloadRangerArtifacts ranger-${RANGER_VERSION}-tagsync.tar.gz ${ranger_url}
}