/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ranger.services.kafka.connect;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.ranger.plugin.model.RangerService;
import org.apache.ranger.plugin.model.RangerServiceDef;
import org.apache.ranger.plugin.service.RangerBaseService;
import org.apache.ranger.plugin.service.ResourceLookupContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class RangerServiceKafkaConnect extends RangerBaseService {

    private static final Log LOG = LogFactory.getLog(RangerServiceKafkaConnect.class);

    @Override
    public void init(RangerServiceDef serviceDef, RangerService service) {
        super.init(serviceDef, service);
    }

    @Override
    public HashMap<String, Object> validateConfig() {
        return new HashMap<>();
    }

    @Override
    public List<String> lookupResource(ResourceLookupContext context) throws Exception {
        final String userInput = context.getUserInput();
        final String resource = context.getResourceName();
        final List<String> resultList = new ArrayList<>();

        if (LOG.isTraceEnabled()) {
            LOG.trace("==> RangerServiceKafkaConnect.lookupResource()  UserInput: \"" + userInput + "\" resource : " + resource);
        }

        if (userInput != null && !userInput.isEmpty() && resource != null) {
            switch (resource.trim().toLowerCase()) {
                case "connector":
                    resultList.add(userInput);
                    break;
                case "cluster":
                    resultList.add("*");
                    break;
                default:
                    break;
            }
        }

        return resultList;
    }

}
