/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.policymigration.common;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;

public class RangerPolicyMigrationConfig extends RangerConfiguration {

	private static final Log LOG = LogFactory.getLog(RangerPolicyMigrationConfig.class);
	private static volatile RangerPolicyMigrationConfig sInstance 	= null;

	public static final String RANGER_POLICYMIGRATION_OUTPUTFILE_PATH          = "ranger.policymigration.transform.output_file";
	public static final String RANGER_POLICYMIGRATION_SERVICE_TYPE_MAPPING     = "ranger.policymigration.service.type.mapping";
	public static final String RANGER_POLICYMIGRATION_SERVICE_NAME_MAPPING     = "ranger.policymigration.service.name.mapping";
	public static final String RANGER_POLICYMIGRATION_ZONE_NAME_MAPPING        = "ranger.policymigration.zone.name.mapping";
	public static final String RANGER_POLICYMIGRATION_SERVICE_RESOURCE_MAPPING = "ranger.policymigration.service.resource.mapping";
	public static final String RANGER_POLICYMIGRATION_S3_BUCKET_NAME           = "ranger.policymigration.s3.bucket.name";
	public static final String RANGER_POLICYMIGRATION_ADLS_STORAGE_ACCOUNT     = "ranger.policymigration.adls.storage.account";
	public static final String RANGER_POLICYMIGRATION_ADLS_STORGAGE_CONTAINER  = "ranger.policymigration.adls.storage.container";
	public static final String RANGER_POLICYMIGRATION_USERS_MAPPING_FILE       = "ranger.policymigration.users.mapping.file";
	public static final String RANGER_POLICYMIGRATION_RESOURCE_MAPPING_FILE    = "ranger.policymigration.resource.mapping.file";
	public static final String RANGER_POLICYMIGRATION_HIVE_URL_MAPPING_FILE    = "ranger.policymigration.hive.url.resource.mapping.file";
	public static final String RANGER_POLICYMIGRATION_EXCLUDE_SERVICE_TYPES    = "ranger.policymigration.exclude.service.types";
	public static final String NATIVE_POLICIES_SERVICE_NAME                    = "ranger.policymigration.nativepolicies.service.name";
	public static final String NATIVE_POLICIES_POLICY_NAME_PREFIX              = "ranger.policymigration.nativepolicies.policy.name.prefix";
	public static final String NATIVE_POLICIES_POLICY_DESCRIPTION              = "ranger.policymigration.nativepolicies.policy.description";

	public static RangerPolicyMigrationConfig getInstance() {
		RangerPolicyMigrationConfig ret = RangerPolicyMigrationConfig.sInstance;

		if (ret == null) {
			synchronized (RangerPolicyMigrationConfig.class) {
				ret = RangerPolicyMigrationConfig.sInstance;

				if (ret == null) {
					ret = RangerPolicyMigrationConfig.sInstance = new RangerPolicyMigrationConfig();
				}
			}
		}

		return ret;
	}

	private RangerPolicyMigrationConfig() {
		super();
		addRangerPolicyMigrationResources();
	}

	private boolean addRangerPolicyMigrationResources() {
		if (LOG.isDebugEnabled()) {
			LOG.debug("==> addRangerPolicyMigrationResources()");
		}

		boolean ret = true;

		if (!addResourceIfReadable(RangerPolicyMigrationConstants.POLICYMIGRATION_CONF)) {
			LOG.error("Could not add " + RangerPolicyMigrationConstants.POLICYMIGRATION_CONF+ " to RangerPolicyMigrationConfig.");
			ret = false;
		}

		if (LOG.isDebugEnabled()) {
			LOG.debug("<== addRangerPolicyMigrationResources(), result=" + ret);
		}

		return ret;
	}

	public static String getProperty(Configuration conf, String property) {
		String value = conf.get(property);
		value = StringUtils.isNotEmpty(value) ? value.trim() : StringUtils.EMPTY;
		return value;
	}
	
}
