/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.ranger.policymigration.common;

import java.util.HashMap;
import java.util.Set;

import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;

import com.google.common.collect.ImmutableSet;

public class RangerPolicyMigrationConstants {

	public static final String LITERAL_HDFS  = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_HDFS_NAME;
	public static final String LITERAL_HIVE  = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_HIVE_NAME;
	public static final String LITERAL_ADLS  = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_ADLS_NAME;
	public static final String LITERAL_S3 	 = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_S3_NAME;
	public static final String LITERAL_STORM = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_STORM_NAME;
	public static final String LITERAL_TAG   = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_TAG_NAME;
	public static final String LITERAL_SOLR  = EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_SOLR_NAME;

	public static final String RESOURCE_TYPE_PATH       = "path";
	public static final String ACCESS_TYPE_EXECUTE      = "execute";
	public static final String ACCESS_TYPE_OTHERS       = "others";
	public static final String ACCESS_TYPE_SOLR_ADMIN   = "solr_admin";
	public static final String ACCESS_TYPE_HDFS_EXECUTE = "hdfs:execute";
	public static final String POLICYMIGRATION_CONF     = "ranger-policymigration-site.xml";

	public static final HashMap<String, Set<String>> NONSUPPORTED_ACCESS_TYPES_MAP         = new HashMap<String, Set<String>>();
	public static final HashMap<String, String>      SERVICE_DEF_ID_NAME_MAP               = new HashMap<String, String>();
	public static final ImmutableSet<String>         TAG_SERVICE_NONSUPPORTED_ACCESS_TYPES = ImmutableSet.<String>builder()
			.add("submitTopology", "fileUpload", "fileDownload", "killTopology", "rebalance","activate",
				 "deactivate", "getTopologyConf", "getTopology", "getUserTopology", "getTopologyInfo", "uploadNewCredentials",
				 "storm:submitTopology", "storm:fileUpload", "storm:fileDownload", "storm:killTopology", "storm:rebalance","storm:activate",
				 "storm:deactivate", "storm:getTopologyConf", "storm:getTopology", "storm:getUserTopology", "storm:getTopologyInfo", "storm:uploadNewCredentials",
				 "others","solr:others","solr_admin","solr:solr_admin")
			.build();

	static {
		NONSUPPORTED_ACCESS_TYPES_MAP.put(LITERAL_HDFS, ImmutableSet.<String>builder().add(ACCESS_TYPE_EXECUTE).build());
		NONSUPPORTED_ACCESS_TYPES_MAP.put(LITERAL_SOLR, ImmutableSet.<String>builder().add(ACCESS_TYPE_OTHERS, ACCESS_TYPE_SOLR_ADMIN).build());
		NONSUPPORTED_ACCESS_TYPES_MAP.put(LITERAL_TAG, TAG_SERVICE_NONSUPPORTED_ACCESS_TYPES);

		SERVICE_DEF_ID_NAME_MAP.put("1", "hdfs");
		SERVICE_DEF_ID_NAME_MAP.put("2", "hbase");
		SERVICE_DEF_ID_NAME_MAP.put("3", "hive");
		SERVICE_DEF_ID_NAME_MAP.put("4", "yarn");
		SERVICE_DEF_ID_NAME_MAP.put("5", "knox");
		SERVICE_DEF_ID_NAME_MAP.put("6", "storm");
		SERVICE_DEF_ID_NAME_MAP.put("7", "kms");
		SERVICE_DEF_ID_NAME_MAP.put("8", "solr");
		SERVICE_DEF_ID_NAME_MAP.put("9", "kafka");
		SERVICE_DEF_ID_NAME_MAP.put("10", "nifi");
		SERVICE_DEF_ID_NAME_MAP.put("12", "kylin");
		SERVICE_DEF_ID_NAME_MAP.put("13", "nifi-registry");
		SERVICE_DEF_ID_NAME_MAP.put("14", "sqoop");
		SERVICE_DEF_ID_NAME_MAP.put("15", "atlas");
		SERVICE_DEF_ID_NAME_MAP.put("100", "tag");
		SERVICE_DEF_ID_NAME_MAP.put("101", "wasb");
		SERVICE_DEF_ID_NAME_MAP.put("103", "abfs");
		SERVICE_DEF_ID_NAME_MAP.put("104", "adls");
		SERVICE_DEF_ID_NAME_MAP.put("105", "kudu");
		SERVICE_DEF_ID_NAME_MAP.put("201", "ozone");
		SERVICE_DEF_ID_NAME_MAP.put("202", "schema-registry");
		SERVICE_DEF_ID_NAME_MAP.put("203", "kafka-connect");
		SERVICE_DEF_ID_NAME_MAP.put("1000", "s3");
	}

}
