package org.apache.ranger.policymigration.model;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.Date;
import java.util.Map;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyResource;
import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

@JsonAutoDetect(fieldVisibility = Visibility.ANY)
@JsonSerialize(include = JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown = true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerPolicyMigrationInfo implements java.io.Serializable {

	private static final long serialVersionUID = 1L;
	private String name;
	private String description;
	private String serviceType;
	private String sourceService;
	private String targetService;
	private String sourceCluster;
	private String targetCluster;
	private String policyStatus;
	private String syncStatus;
	private Map<String, RangerPolicyResource> sourceDataSet;
	private Map<String, RangerPolicyResource> targetDataSet;
	private Date startTime;
	private Date endTime;
	private Long frequency;
	private Long version;

	public RangerPolicyMigrationInfo() {
		this(null, null, null, null, null, null, null, null, null, null, null, null, null, null, -1L);
	}

	public RangerPolicyMigrationInfo(String name, String serviceType, String sourceService, String targetService,
			String policyStatus, String syncStatus, Date startTime, Date endTime, Long frequency, String Description,
			Map<String, RangerPolicyResource> sourceResources, Map<String, RangerPolicyResource> targetResources, String srcCluster, String targetCluster, Long version) {
		setName(name);
		setServiceType(serviceType);
		setSourceService(sourceService);
		setTargetService(targetService);
		setPolicyStatus(policyStatus);
		setSyncStatus(syncStatus);
		setStartTime(startTime);
		setEndTime(endTime);
		setFrequency(frequency);
		setDescription(Description);
		setSourceDataSet(sourceResources);
		setTargetDataSet(targetResources);
		setSourceCluster(srcCluster);
		setTargetCluster(targetCluster);
		this.version = version;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getServiceType() {
		return serviceType;
	}

	public void setServiceType(String serviceType) {
		if (serviceType != null) {
			this.serviceType = serviceType.toLowerCase();
		}
	}

	public String getSourceService() {
		return sourceService;
	}

	public void setSourceService(String sourceService) {
		this.sourceService = sourceService;
	}

	public String getTargetService() {
		return targetService;
	}

	public void setTargetService(String targetService) {
		this.targetService = targetService;
	}

	public String getPolicyStatus() {
		return policyStatus;
	}

	public void setPolicyStatus(String policyStatus) {
		this.policyStatus = policyStatus;
	}

	public String getSyncStatus() {
		return syncStatus;
	}

	public void setSyncStatus(String syncStatus) {
		this.syncStatus = syncStatus;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public Long getFrequency() {
		return frequency;
	}

	public void setFrequency(Long frequency) {
		this.frequency = frequency;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Map<String, RangerPolicyResource> getSourceDataSet() {
		return sourceDataSet;
	}

	public void setSourceDataSet(Map<String, RangerPolicyResource> sourceDataSet) {
		this.sourceDataSet = sourceDataSet;
	}

	public Map<String, RangerPolicyResource> getTargetDataSet() {
		return targetDataSet;
	}

	public void setTargetDataSet(Map<String, RangerPolicyResource> targetDataSet) {
		this.targetDataSet = targetDataSet;
	}

	public String getSourceCluster() {
		return sourceCluster;
	}

	public void setSourceCluster(String sourceCluster) {
		this.sourceCluster = sourceCluster;
	}

	public String getTargetCluster() {
		return targetCluster;
	}

	public void setTargetCluster(String targetCluster) {
		this.targetCluster = targetCluster;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerPolicyMigrationInfo={");
		sb.append("name={").append(name).append("} ");
		sb.append("serviceType={").append(serviceType).append("} ");
		sb.append("sourceService={").append(sourceService).append("} ");
		sb.append("targetService={").append(targetService).append("} ");
		sb.append("policyStatus={").append(policyStatus).append("} ");
		sb.append("syncStatus={").append(syncStatus).append("} ");
		sb.append("startTime={").append(startTime).append("} ");
		sb.append("endTime={").append(endTime).append("} ");
		sb.append("frequency={").append(frequency).append("} ");
		sb.append("Description={").append(description).append("} ");
		sb.append("Version={").append(version).append("} ");
		sb.append("resources={");
		if (sourceDataSet != null) {
			for (Map.Entry<String, RangerPolicyResource> e : sourceDataSet.entrySet()) {
				sb.append(e.getKey()).append("={");
				e.getValue().toString(sb);
				sb.append("} ");
			}
		}
		sb.append("targetResources={");
		if (targetDataSet != null) {
			for (Map.Entry<String, RangerPolicyResource> e : targetDataSet.entrySet()) {
				sb.append(e.getKey()).append("={");
				e.getValue().toString(sb);
				sb.append("} ");
			}
		}
		sb.append("}");
		return sb;
	}
}
