/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.hook.s3;

import org.apache.hadoop.io.Text;
import org.apache.hadoop.security.token.delegation.web.DelegationTokenIdentifier;

/**
 * This class will be used for parsing the delegation
 * token obtained from RAZ service through call
 * {@link org.apache.ranger.raz.intg.client.RangerRazClient#getDelegationToken(String)}
 * and converting it to {@link RazToken}. This gets instantiated through
 * service loader in {@code Token#decodeIdentifier()}.
 */
public class RazS3ADelegationTokenIdentifier extends DelegationTokenIdentifier {
    static final Text TOKEN_KIND = new Text("S3 delegation");

    public RazS3ADelegationTokenIdentifier() {
        super(TOKEN_KIND);
    }

    public RazS3ADelegationTokenIdentifier(Text kind) {
        super(kind);
    }

    public RazS3ADelegationTokenIdentifier(Text kind, Text owner, Text renewer, Text realUser) {
        super(kind, owner, renewer, realUser);
    }

    public Text getKind() {
        return TOKEN_KIND;
    }
}
