/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.hook.s3;

import org.apache.hadoop.fs.s3a.auth.delegation.AbstractS3ATokenIdentifier;
import org.apache.hadoop.fs.s3a.auth.delegation.EncryptionSecrets;
import org.apache.hadoop.io.Text;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.net.URI;

/**
 * Raz Token identifier for S3A: RAZ token and an expiry time of the RAZ token.
 *
 */
public class RazS3ATokenIdentifier extends AbstractS3ATokenIdentifier {

  private RazToken razToken;

  private static final long serialVersionUID = 4L;

  /**
   * Name of token: {@value}.
   */
  static final String RAZ_TOKEN_NAME = "S3ADelegationToken/Raz";

  /**
   * Kind of token; value is {@link #RAZ_TOKEN_NAME}.
   */
  static final Text RAZ_TOKEN_KIND = new Text(RAZ_TOKEN_NAME);

  /**
   * Constructor for service loader use.
   * Subclasses MUST NOT subclass this; they must provide their own
   * token kind.
   */
  public RazS3ATokenIdentifier() {
    super(RAZ_TOKEN_KIND);
  }

  public RazS3ATokenIdentifier(
      final Text kind,
      final Text owner,
      final Text renewer,
      final URI uri,
      final String origin,
      final EncryptionSecrets encryptionSecrets,
      final RazToken razToken
  ) {
    super(kind, uri, owner, renewer, origin, encryptionSecrets);

    this.razToken = razToken;
  }

  @Override
  public void write(final DataOutput out) throws IOException {
    super.write(out);
    razToken.write(out);
  }

  @Override
  public void readFields(final DataInput in)
      throws IOException {
    super.readFields(in);
    this.razToken = new RazToken();
    this.razToken.readFields(in);
  }

  @Override
  public long getExpiryTime() {
    return razToken.getExpiryTime();
  }

  public RazToken getRazToken() {
    return razToken;
  }
}
