/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.hook.s3;

import junit.framework.TestCase;
import org.junit.Assert;
import org.junit.Test;

import java.time.Instant;

import static java.time.temporal.ChronoUnit.SECONDS;

public class TestRazToken extends TestCase {

  @Test
  public void testIsAboutToExpireFalse() {
    long notExpired = 100;
    long offset = 1;

    RazToken razToken = new RazToken(
        Instant.now().getEpochSecond(),
        Instant.now().getEpochSecond() + notExpired,
        "");

    Assert.assertFalse("The token should not be expired.",
        razToken.isAboutToExpire(
            razToken.getExpiryTime(),
            offset,
            SECONDS
        )
    );
  }

  @Test
  public void testIsAboutToExpireTrue() {
    long expired = -10;
    long offset = 1;

    RazToken razToken = new RazToken(
        Instant.now().getEpochSecond(),
        Instant.now().getEpochSecond() + expired,
        "");

    Assert.assertTrue("The token should be expired.",
        razToken.isAboutToExpire(
            razToken.getExpiryTime(),
            offset,
            SECONDS)
    );
  }
}