/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.intg.client;

import org.slf4j.Logger;


public class RangerRazClientLogger {
	private static boolean QUIET_MODE = true;

	public static void setIsQuietMode(boolean isQuietMode) {
		QUIET_MODE = isQuietMode;
	}

	public static void warn(Logger log, String msg) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg);
			}
		} else {
			log.warn(msg);
		}
	}

	public static void warn(Logger log, String msg, Throwable t) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg, t);
			}
		} else {
			log.warn(msg, t);
		}
	}

	public static void warn(Logger log, String format, Object arg1) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1);
			}
		} else {
			log.warn(format, arg1);
		}
	}

	public static void warn(Logger log, String format, Object arg1, Object arg2) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1, arg2);
			}
		} else {
			log.warn(format, arg1, arg2);
		}
	}

	public static void warn(Logger log, String format, Object...args) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, args);
			}
		} else {
			log.warn(format, args);
		}
	}


	public static void error(Logger log, String msg) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg);
			}
		} else {
			log.error(msg);
		}
	}

	public static void error(Logger log, String msg, Throwable t) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg, t);
			}
		} else {
			log.error(msg, t);
		}
	}

	public static void error(Logger log, String format, Object arg1) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1);
			}
		} else {
			log.error(format, arg1);
		}
	}

	public static void error(Logger log, String format, Object arg1, Object arg2) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1, arg2);
			}
		} else {
			log.error(format, arg1, arg2);
		}
	}

	public static void error(Logger log, String format, Object...args) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, args);
			}
		} else {
			log.error(format, args);
		}
	}


	public static void info(Logger log, String msg) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg);
			}
		} else {
			log.info(msg);
		}
	}

	public static void info(Logger log, String msg, Throwable t) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(msg, t);
			}
		} else {
			log.info(msg, t);
		}
	}

	public static void info(Logger log, String format, Object arg1) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1);
			}
		} else {
			log.info(format, arg1);
		}
	}

	public static void info(Logger log, String format, Object arg1, Object arg2) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, arg1, arg2);
			}
		} else {
			log.info(format, arg1, arg2);
		}
	}

	public static void info(Logger log, String format, Object...args) {
		if (QUIET_MODE) {
			if (log.isDebugEnabled()) {
				log.debug(format, args);
			}
		} else {
			log.info(format, args);
		}
	}
}
