/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections.MapUtils;

public class RangerRazIdentityMappingsUtil {
	private final Map<String, String> userClusterIdToCloudId  = new HashMap<>();
	private final Map<String, String> userCloudIdToClusterId  = new HashMap<>();
	private final Map<String, String> groupClusterIdToCloudId = new HashMap<>();
	private final Map<String, String> groupCloudIdToClusterId = new HashMap<>();
	private final Long version;

	public RangerRazIdentityMappingsUtil(RangerRazIdentityMappings idMappings) {
		//User
		if (MapUtils.isNotEmpty(idMappings.getUserClusterIdToCloudId())) {
			this.userClusterIdToCloudId.putAll(idMappings.getUserClusterIdToCloudId());
		}

		for (Map.Entry<String, String> entry : this.userClusterIdToCloudId.entrySet()) {
			this.userCloudIdToClusterId.put(entry.getValue(), entry.getKey());
		}

		//Group
		if (MapUtils.isNotEmpty(idMappings.getGroupClusterIdToCloudId())) {
			this.groupClusterIdToCloudId.putAll(idMappings.getGroupClusterIdToCloudId());
		}

		for (Map.Entry<String, String> entry : this.groupClusterIdToCloudId.entrySet()) {
			this.groupCloudIdToClusterId.put(entry.getValue(), entry.getKey());
		}

		this.version = idMappings.getVersion();
	}

	public String getUserCloudId(String clusterId) {
		return userClusterIdToCloudId.get(clusterId);
	}

	public String getUserClusterId(String cloudId) {
		return userCloudIdToClusterId.get(cloudId);
	}

	public String getGroupCloudId(String groupClusterId) {
		return groupClusterIdToCloudId.get(groupClusterId);
	}

	public String getGroupClusterId(String groupCloudId) {
		return groupCloudIdToClusterId.get(groupCloudId);
	}

	public Long getVersion() {
		return version;
	}
}
