/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@JsonAutoDetect(fieldVisibility=Visibility.ANY)
@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown=true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerRazMultiOperationRequest extends RangerRazRequestBase implements java.io.Serializable {
	private static final long serialVersionUID = 1L;

	private List<ResourceAccess> operations;


	public RangerRazMultiOperationRequest() { }

	public RangerRazMultiOperationRequest(String serviceType, ResourceAccess operation, String user) {
		setServiceType(serviceType);
		setUser(user);
		setAccessTime(new Date());

		addOperation(operation);
	}

	public List<ResourceAccess> getOperations() {
		return operations;
	}

	public void setOperations(List<ResourceAccess> operations) {
		this.operations = operations;
	}

	@JsonIgnore
	public void addOperation(ResourceAccess operation) {
		List<ResourceAccess> operations = this.operations;

		if (operations == null) {
			this.operations = operations = new ArrayList<>();
		}

		operations.add(operation);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerRazMultiOperationRequest={");

		super.toString(sb);

		sb.append("operations=[");
		if (operations != null) {
			for (ResourceAccess operation : operations) {
				operation.toString(sb).append(" ");
			}
		}
		sb.append("]");

		sb.append("}");

		return sb;
	}
}
