/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@JsonAutoDetect(fieldVisibility=Visibility.ANY)
@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown=true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerRazMultiOperationResult extends RangerRazResultBase implements java.io.Serializable {
	private static final long serialVersionUID = 1L;

	private List<ResourceAccessResult> operResults;

	public RangerRazMultiOperationResult() { }

	public RangerRazMultiOperationResult(String requestId) {
		super(requestId);
	}

	public RangerRazMultiOperationResult(String requestId, ResourceAccessResult operResult) {
		super(requestId);

		addOperResult(operResult);
	}

	public RangerRazMultiOperationResult(String requestId, AccessResult result, Boolean isAudited, List<AuditInfo> auditLogs, Map<String, String> additionalInfo) {
		super(requestId);

		addOperResult(new ResourceAccessResult(result, isAudited, auditLogs, additionalInfo));
	}

	public List<ResourceAccessResult> getOperResults() {
		return operResults;
	}

	public void setResults(List<ResourceAccessResult> operResults) {
		this.operResults = operResults;
	}

	@JsonIgnore
	public void addOperResult(ResourceAccessResult operResult) {
		List<ResourceAccessResult> operResults = this.operResults;

		if (operResults == null) {
			this.operResults = operResults = new ArrayList<>();
		}

		operResults.add(operResult);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerRazMultiOperationResult={");

		super.toString(sb);

		sb.append("operResults=[");
		if (operResults != null) {
			for (ResourceAccessResult operResult : operResults) {
				operResult.toString(sb).append(" ");
			}
		}
		sb.append("] ");

		sb.append("}");

		return sb;
	}
}
