/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.Date;


@JsonAutoDetect(fieldVisibility=Visibility.ANY)
@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown=true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerRazRequest extends RangerRazRequestBase implements java.io.Serializable {
	private static final long serialVersionUID = 1L;

	private ResourceAccess operation;


	public RangerRazRequest() { }

	public RangerRazRequest(String serviceType, ResourceAccess operation, String user) {
		setServiceType(serviceType);
		setUser(user);
		setAccessTime(new Date());

		setOperation(operation);
	}

	public RangerRazRequest(RangerRazRequestBase requestBase, ResourceAccess operation) {
		super(requestBase);

		setOperation(operation);
	}

	public ResourceAccess getOperation() {
		return operation;
	}

	public void setOperation(ResourceAccess operation) {
		this.operation = operation;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerRazRequest={");

		super.toString(sb);

		sb.append("operation={");
		if (operation != null) {
			operation.toString(sb);
		}
		sb.append("}");

		sb.append("}");

		return sb;
	}
}
