/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import static org.apache.ranger.raz.intg.client.RangerRazClient.MASKED_TOKEN_STR;

import org.apache.commons.collections.CollectionUtils;
import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.Date;
import java.util.Map;
import java.util.Set;


@JsonAutoDetect(fieldVisibility=Visibility.ANY)
@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown=true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerRazRequestBase implements java.io.Serializable {
	private static final long serialVersionUID = 1L;

	private String               requestId;
	private String               serviceType;
	private String               serviceName;
	private String               user;
	private Set<String>          userGroups;
	private Date                 accessTime;
	private String               clientIpAddress;
	private String               clientType;
	private String               clusterName;
	private String               clusterType;
	private String               sessionId;
	private Set<String>          keysToRedact;
	private Map<String, String>  context;


	public RangerRazRequestBase() { }

	public RangerRazRequestBase(RangerRazRequestBase other) {
		this.requestId       = other.requestId;
		this.serviceType     = other.serviceType;
		this.serviceName     = other.serviceName;
		this.user            = other.user;
		this.userGroups      = other.userGroups;
		this.accessTime      = other.accessTime;
		this.clientIpAddress = other.clientIpAddress;
		this.clientType      = other.clientType;
		this.clusterName     = other.clusterName;
		this.clusterType     = other.clusterType;
		this.sessionId       = other.sessionId;
		this.context         = other.context;
		this.keysToRedact    = other.keysToRedact;
	}

	public String getRequestId() { return requestId; }

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

    public String getServiceType() {
        return serviceType;
    }

    public void setServiceType(String serviceType) {
        this.serviceType = serviceType;
    }

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
		this.serviceName = serviceName;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public Set<String> getUserGroups() {
		return userGroups;
	}

	public void setUserGroups(Set<String> userGroups) {
		this.userGroups = userGroups;
	}

	public Date getAccessTime() {
		return accessTime;
	}

	public void setAccessTime(Date accessTime) {
		this.accessTime = accessTime;
	}

	public String getClientIpAddress() {
		return clientIpAddress;
	}

	public void setClientIpAddress(String clientIpAddress) {
		this.clientIpAddress = clientIpAddress;
	}

	public String getClientType() {
		return clientType;
	}

	public void setClientType(String clientType) {
		this.clientType = clientType;
	}

	public String getClusterName() {
		return clusterName;
	}

	public void setClusterName(String clusterName) {
		this.clusterName = clusterName;
	}

	public String getClusterType() {
		return clusterType;
	}

	public void setClusterType(final String clusterType) {
		this.clusterType = clusterType;
	}

	public String getSessionId() {
		return sessionId;
	}

	public void setSessionId(String sessionId) {
		this.sessionId = sessionId;
	}

	public Map<String, String> getContext() {
		return context;
	}

	public void setContext(Map<String, String> context) {
		this.context = context;
	}

	public Set<String> getKeysToRedact() {
		return keysToRedact;
	}

	public void setKeysToRedact(Set<String> keysToRedact) {
		this.keysToRedact = keysToRedact;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerRazRequest={");

        sb.append("requestId={").append(requestId).append("} ");
        sb.append("serviceType={").append(serviceType).append("} ");
		sb.append("serviceName={").append(serviceName).append("} ");
		sb.append("user={").append(user).append("} ");
		sb.append("userGroups=[");
		if (userGroups != null) {
			for (String userGroup : userGroups) {
				sb.append(userGroup).append(" ");
			}
		}
		sb.append("] ");
		sb.append("accessTime={").append(accessTime).append("} ");
		sb.append("clientIpAddress={").append(clientIpAddress).append("} ");
		sb.append("clientType={").append(clientType).append("} ");
		sb.append("clusterName={").append(clusterName).append("} ");
		sb.append("clusterType={").append(clusterType).append("} ");
		sb.append("sessionId={").append(sessionId).append("} ");
		sb.append("context={");
		if (context != null) {
			Set<String> redactKeys = getKeysToRedact();
			for (Map.Entry<String, String> e : context.entrySet()) {
				String key = e.getKey();
				sb.append(key).append("={");
				sb.append((CollectionUtils.isNotEmpty(redactKeys) && redactKeys.contains(key)) ? MASKED_TOKEN_STR : e.getValue());
				sb.append("} ");
			}
		}
		sb.append("} ");

		sb.append("}");

		return sb;
	}


	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
	@JsonIgnoreProperties(ignoreUnknown=true)
	@XmlRootElement
	@XmlAccessorType(XmlAccessType.FIELD)
	public static class ResourceAccess implements java.io.Serializable {
		private static final long serialVersionUID = 1L;

		private Map<String, String> resource;
		private String              resourceOwner;
		private String              action;
		private Set<String>         accessTypes;

		public ResourceAccess() {
		}

		public ResourceAccess(Map<String, String> resource, String action) {
			this.resource = resource;
			this.action   = action;
		}

		public ResourceAccess(Map<String, String> resource, String resourceOwner, String action, Set<String> accessTypes) {
			this.resource      = resource;
			this.resourceOwner = resourceOwner;
			this.action        = action;
			this.accessTypes   = accessTypes;
		}

		public Map<String, String> getResource() {
			return resource;
		}

		public void setResource(Map<String, String> resource) {
			this.resource = resource;
		}

		public String getResourceOwner() {
			return resourceOwner;
		}

		public void setResourceOwner(String resourceOwner) {
			this.resourceOwner = resourceOwner;
		}

		public String getAction() {
			return action;
		}

		public void setAction(String action) {
			this.action = action;
		}

		public Set<String> getAccessTypes() {
			return accessTypes;
		}

		public void setAccessTypes(Set<String> accessTypes) {
			this.accessTypes = accessTypes;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			toString(sb);

			return sb.toString();
		}

		public StringBuilder toString(StringBuilder sb) {
			sb.append("ResourceAccess={");

			sb.append("resource={");
			if (resource != null) {
				for (Map.Entry<String, String> e : resource.entrySet()) {
					sb.append(e.getKey()).append("=");
					sb.append(e.getValue());
					sb.append(" ");
				}
			}
			sb.append("} ");
			sb.append("resourceOwner={").append(resourceOwner).append("} ");
			sb.append("action={").append(action).append("} ");
			sb.append("accessTypes=[");
			if (accessTypes != null) {
				for (String accessType : accessTypes) {
					sb.append(accessType).append(" ");
				}
			}
			sb.append("] ");

			sb.append("}");

			return sb;
		}
	}
}
