/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.model;

import static org.apache.ranger.raz.intg.client.RangerRazClient.MASKED_TOKEN_STR;

import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
import org.codehaus.jackson.annotate.JsonIgnoreProperties;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

import java.util.List;
import java.util.Map;
import java.util.Set;


@JsonAutoDetect(fieldVisibility=Visibility.ANY)
@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
@JsonIgnoreProperties(ignoreUnknown=true)
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class RangerRazResultBase implements java.io.Serializable {
	private static final long serialVersionUID = 1L;

	public enum AccessResult { ALLOWED, DENIED, NOT_DETERMINED }

	private String requestId;

	public RangerRazResultBase() { }

	public RangerRazResultBase(String requestId) {
		setRequestId(requestId);
	}

	public String getRequestId() {
		return requestId;
	}

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("requestId={").append(requestId).append("} ");

		return sb;
	}


	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
	@JsonIgnoreProperties(ignoreUnknown=true)
	@XmlRootElement
	@XmlAccessorType(XmlAccessType.FIELD)
	public static class ResourceAccessResult implements java.io.Serializable {
		private static final long serialVersionUID = 1L;

		private AccessResult        result;
		private Boolean             isAudited;
		private List<AuditInfo>     auditLogs;
		private Map<String, String> additionalInfo;
		private Set<String>         keysToRedact;

		public ResourceAccessResult() { }

		public ResourceAccessResult(AccessResult result, Boolean isAudited, List<AuditInfo> auditLogs, Map<String, String> additionalInfo) {
			this(result, isAudited, auditLogs ,additionalInfo, null);
		}

		public ResourceAccessResult(AccessResult result, Boolean isAudited, List<AuditInfo> auditLogs, Map<String, String> additionalInfo, Set<String> keysToRedact) {
			this.result         = result;
			this.isAudited      = isAudited;
			this.auditLogs      = auditLogs;
			this.additionalInfo = additionalInfo;
			this.keysToRedact   = keysToRedact;
		}

		public AccessResult getResult() {
			return result;
		}

		public void setResult(AccessResult result) {
			this.result = result;
		}

		public Boolean getIsAudited() {
			return isAudited;
		}

		public void setIsAudited(Boolean audited) {
			isAudited = audited;
		}

		public List<AuditInfo> getAuditLogs() {
			return auditLogs;
		}

		public void setAuditLogs(List<AuditInfo> auditLogs) {
			this.auditLogs = auditLogs;
		}

		public Map<String, String> getAdditionalInfo() {
			return additionalInfo;
		}

		public void setAdditionalInfo(Map<String, String> additionalInfo) {
			this.additionalInfo = additionalInfo;
		}

		public Set<String> getKeysToRedact() {
			return keysToRedact;
		}

		public void setKeysToRedact(Set<String> keysToRedact) {
			this.keysToRedact = keysToRedact;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			toString(sb);

			return sb.toString();
		}

		public StringBuilder toString(StringBuilder sb) {
			sb.append("ResourceAccessResult={");

			sb.append("result={").append(result).append("} ");
			sb.append("isAudited={").append(isAudited).append("} ");
			sb.append("auditLogs={");
			if (auditLogs != null) {
				for (AuditInfo auditLog : auditLogs) {
					auditLog.toString(sb);
					sb.append(" ");
				}
			}
			sb.append("} ");
			sb.append("additionalInfo={");
			if (additionalInfo != null) {
				for (Map.Entry<String, String> e : additionalInfo.entrySet()) {
					String key = e.getKey();
					sb.append(key).append("={");
					sb.append(keysToRedact.contains(key) ? MASKED_TOKEN_STR : sb.append(e.getValue()));
					sb.append("} ");
				}
			}
			sb.append("} ");

			sb.append("}");

			return sb;
		}
	}

	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
	@JsonIgnoreProperties(ignoreUnknown=true)
	@XmlRootElement
	@XmlAccessorType(XmlAccessType.FIELD)
	public static class AuditInfo implements java.io.Serializable {
		private static final long serialVersionUID = 1L;

		private String       auditId;
		private String       accessType;
		private AccessResult result;
		private Long         policyId;
		private Long         policyVersion;

		public AuditInfo() { }

		public AuditInfo(String auditId, String accessType, AccessResult result, Long policyId, Long policyVersion) {
			setAuditId(auditId);
			setAccessType(accessType);
			setResult(result);
			setPolicyId(policyId);
			setPolicyVersion(policyVersion);
		}

		public String getAuditId() {
			return auditId;
		}

		public void setAuditId(String auditId) {
			this.auditId = auditId;
		}

		public String getAccessType() {
			return accessType;
		}

		public void setAccessType(String accessType) {
			this.accessType = accessType;
		}

		public AccessResult getResult() {
			return result;
		}

		public void setResult(AccessResult result) {
			this.result = result;
		}

		public Long getPolicyId() {
			return policyId;
		}

		public void setPolicyId(Long policyId) {
			this.policyId = policyId;
		}

		public Long getPolicyVersion() {
			return policyVersion;
		}

		public void setPolicyVersion(Long policyVersion) {
			this.policyVersion = policyVersion;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			toString(sb);

			return sb.toString();
		}

		public StringBuilder toString(StringBuilder sb) {
			sb.append("AuditInfo={");

			sb.append("auditId={").append(auditId).append("} ");
			sb.append("accessType={").append(accessType).append("} ");
			sb.append("result={").append(result).append("} ");
			sb.append("policyId={").append(policyId).append("} ");
			sb.append("policyVersion={").append(policyVersion).append("} ");

			sb.append("}");

			return sb;
		}
	}
}
