/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.processor;

import org.apache.ranger.plugin.util.RangerUserStore;

import java.util.Objects;
import java.util.Set;

public class RangerRazRequestContext {
	private final String            remoteUser;
	private final Set<String>       remoteUserGroups;
	private final String            remoteIpAddress;
	private final String[]          remoteForwadedAddresses;
	private RangerUserStore         rangerUserStore;
	private Set<String>             keysToRedact;

	public RangerRazRequestContext(String remoteUser, Set<String> remoteUserGroups, String remoteIpAddress, String[] remoteForwadedAddresses, RangerUserStore rangerUserStore, Set<String> keysToRedact) {
		this.remoteUser              = remoteUser;
		this.remoteUserGroups        = remoteUserGroups;
		this.remoteIpAddress         = remoteIpAddress;
		this.remoteForwadedAddresses = remoteForwadedAddresses;
		this.rangerUserStore         = rangerUserStore;
		this.keysToRedact            = keysToRedact;
	}

	public RangerRazRequestContext(String remoteUser, Set<String> remoteUserGroups, String remoteIpAddress, String[] remoteForwadedAddresses, RangerUserStore rangerUserStore) {
		this(remoteUser, remoteUserGroups, remoteIpAddress, remoteForwadedAddresses, rangerUserStore, null);
	}

	public RangerRazRequestContext(String remoteUser, Set<String> remoteUserGroups, String remoteIpAddress, String[] remoteForwadedAddresses) {
		this(remoteUser, remoteUserGroups, remoteIpAddress, remoteForwadedAddresses, null);
	}

	public String getRemoteUser() {
		return remoteUser;
	}

	public Set<String> getRemoteUserGroups() { return remoteUserGroups; }

	public String getRemoteIpAddress() {
		return remoteIpAddress;
	}

	public String[] getRemoteForwadedAddresses() {
		return remoteForwadedAddresses;
	}

	public RangerUserStore getRangerUserStore() {
		return rangerUserStore;
	}

	public void setRangerUserStore(RangerUserStore rangerUserStore) {
		this.rangerUserStore = rangerUserStore;
	}

	public Set<String> getKeysToRedact() {
		return keysToRedact;
	}

	public void setKeysToRedact(Set<String> keysToRedact) {
		this.keysToRedact = keysToRedact;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		toString(sb);

		return sb.toString();
	}

	@Override
	public int hashCode() {
		return Objects.hash(remoteUser, remoteIpAddress, remoteForwadedAddresses);
	}

	public StringBuilder toString(StringBuilder sb) {
		sb.append("RangerRazRequestContext={");

		sb.append("remoteUser={").append(remoteUser).append("} ");
		sb.append("remoteUserGroups={");
		if (remoteUserGroups != null) {
			for (String remoteUserGroup : remoteUserGroups) {
				sb.append(remoteUserGroup).append(" ");
			}
		}
		sb.append("} ");
		sb.append("remoteIpAddress={").append(remoteIpAddress).append("} ");
		sb.append("remoteForwadedAddresses={");
		if (remoteForwadedAddresses != null) {
			for (String remoteForwadedAddresse : remoteForwadedAddresses) {
				sb.append(remoteForwadedAddresse).append(" ");
			}
		}
		sb.append("} ");

		sb.append("}");

		return sb;
	}
}
