/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.exceptions;

import com.amazonaws.AmazonServiceException;

public class RCSAWSException extends AmazonServiceException {

  public enum StatusCode {

    ACCESS_DENIED(403),
    BAD_REQUEST(400),
    THROTTLED(503),
    INTERNAL_ERROR(500),

    // Intentionally using an invalid code. Most exceptions thrown by RCSClient will fall under this
    // code
    OTHER(600),
    ;


    private final int code;

    StatusCode(int code) {
      this.code = code;
    }

    public int getCode() {
      return code;
    }
  }

  public RCSAWSException(StatusCode statusCode, String errorMessage) {
    super(errorMessage);
    setStatusCode(statusCode.getCode());
  }

  public RCSAWSException(StatusCode statusCode, String errorMessage, Exception cause) {
    super(errorMessage, cause);
    setStatusCode(statusCode.getCode());
  }

  public RCSAWSException(StatusCode statusCode, Exception cause) {
    this(statusCode, null, cause);
  }
}
