/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.fsrequests;

import com.amazonaws.Request;

import org.apache.ranger.raz.s3.lib.RazS3Constants.AccessType;
import org.apache.ranger.raz.s3.lib.aws.parsers.RequestBodyParser;
import org.apache.ranger.raz.s3.lib.aws.parsers.RequestBodyParser.DeleteObjects;
import org.apache.ranger.raz.s3.lib.signer.ContentSigningRequired;
import com.google.common.base.Preconditions;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.List;
import java.util.Map;

@ContentSigningRequired(true)
public class AwsDeleteObjectsRequest extends AwsRequest {

  private AwsDeleteObjectsRequest(URI endpoint,
      Map<String, String> headers,
      Map<String, List<String>> parameters, String[] resourcePaths) {
    super(AwsOperationType.DELETE_OBJECTS, endpoint, headers, parameters, resourcePaths,
        AccessType.WRITE);
  }

  public static AwsDeleteObjectsRequest create(Request<?> request) throws IOException {
    InputStream is = request.getContent();

    // Preconditions.checkNotNull(is, "Content for delete request is null");
    // TODO fix this in S3A and then remove it!
    if (is == null) {
      is = new ByteArrayInputStream("<Delete></Delete>".getBytes());
    }

    Preconditions.checkState(is.markSupported(),
        "mark not supported on delete request stream. Parsing the contents here will cause subsequent failure when the contents are consumed");

    is.mark(Integer.MAX_VALUE);

    RequestBodyParser requestBodyParser = new RequestBodyParser();
    try {
      DeleteObjects deleteObjects = requestBodyParser.parseDeleteObjectsRequest(is);
      String[] paths = deleteObjects.getDeleteEntities().stream()
          .map(deleteObject -> deleteObject.getKey()).toArray(size -> new String[size]);
      return new AwsDeleteObjectsRequest(request.getEndpoint(), request.getHeaders(),
          request.getParameters(), paths);
    } finally {
      is.reset();
    }

  }
}
