/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.fsrequests;

import com.amazonaws.Request;
import org.apache.ranger.raz.s3.lib.RazS3Constants.AccessType;
import org.apache.ranger.raz.s3.lib.aws.S3Parameters;
import com.google.common.base.Preconditions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.util.List;
import java.util.Map;

public class AwsListV1Request extends AwsRequest {

  private static final Logger LOG = LoggerFactory.getLogger(AwsListV1Request.class.getName());

  private AwsListV1Request(URI endpoint,
      Map<String, String> headers,
      Map<String, List<String>> parameters, String resourcePath) {
    super(AwsOperationType.LIST_V1, endpoint, headers, parameters, resourcePath,
        AccessType.READ);
  }

  public static AwsListV1Request create(Request<?> request) {
    List<String> prefixParams = request.getParameters().get(S3Parameters.PREFIX);
    if (prefixParams == null || prefixParams.size() == 0) {
      LOG.debug("Prefix Params is null or empty. Assuming bucket root list: {}", prefixParams);
    }
    Preconditions.checkState(prefixParams == null || prefixParams.size() == 0 || prefixParams.size() == 1,
            S3Parameters.PREFIX + " should be null, or containt 0/1 argument");

    String resourcePath = (prefixParams == null || prefixParams.size() == 0) ? "" : prefixParams.get(0);
    AwsListV1Request listV1Request = new AwsListV1Request(request.getEndpoint(),
        request.getHeaders(), request.getParameters(), resourcePath);
    return listV1Request;

  }
}
