/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.fsrequests;

import com.amazonaws.Request;

import org.apache.ranger.raz.s3.lib.RazS3Constants.AccessType;
import org.apache.ranger.raz.s3.lib.aws.S3Parameters;
import com.google.common.base.Preconditions;

import java.net.URI;
import java.util.List;
import java.util.Map;

public class AwsMultiPartListUploadsRequest extends AwsRequest {

  private AwsMultiPartListUploadsRequest(URI endpoint, Map<String, String> headers,
      Map<String, List<String>> parameters, String resourcePath) {
    super(AwsOperationType.MULTIPART_LIST, endpoint, headers, parameters, resourcePath,
        AccessType.READ);
  }

  public static AwsMultiPartListUploadsRequest create(Request<?> request) {
    List<String> prefixParams = request.getParameters().get(S3Parameters.PREFIX);
    Preconditions.checkState(prefixParams != null && prefixParams.size() == 1,
        S3Parameters.PREFIX + " should have exactly one value");
    return new AwsMultiPartListUploadsRequest(request.getEndpoint(), request.getHeaders(),
        request.getParameters(), prefixParams.get(0));
  }
}
