/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.fsrequests;

import com.amazonaws.http.HttpMethodName;

// TODO: It may be possible to make this generic to span across cloud providers. The operations
// and associated HttpMethods could be different though - in which case they can be split.
public enum AwsOperationType {
  HEAD_BUCKET_METADATA(HttpMethodName.HEAD),
  HEAD_OBJECT_METADATA(HttpMethodName.HEAD),
  LIST_V2(HttpMethodName.GET),
  READ_OBJECT(HttpMethodName.GET),
  PUT_OBJECT(HttpMethodName.PUT),
  DELETE_OBJECTS(HttpMethodName.POST),
  DELETE_OBJECT(HttpMethodName.DELETE),
  MULTIPART_INITIATE(HttpMethodName.POST),
  MULTIPART_UPLOAD(HttpMethodName.PUT),
  MULTIPART_COMMIT(HttpMethodName.POST),
  MULTIPART_LIST(HttpMethodName.GET),
  MULTIPART_LIST_PARTS(HttpMethodName.GET),
  MULTIPART_ABORT(HttpMethodName.DELETE), //Not seen yet via S3A
  LIST_V1(HttpMethodName.GET),
  GET_BUCKET_LOCATION(HttpMethodName.GET),
  GET_BUCKET_ACL(HttpMethodName.GET), // Used for bucket existence check. Vital.
  READ_SELECTED(HttpMethodName.POST), //Not seen yet via S3A
  ;


  private final HttpMethodName httpMethodName;

  AwsOperationType(HttpMethodName httpMethodName) {
    this.httpMethodName = httpMethodName;
  }

  public HttpMethodName getHttpMethodName() {
    return httpMethodName;
  }

}
