/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.exceptions;

public class RCSUncheckedException extends RuntimeException {

  // Maybe include information on the action the client can take in case of such errors.
  // Example, back off and try again later. This is if and when required.

  public enum Code {

    GENERAL_SERVER_ERROR(1),

    // This is only thrown for bucket lookup at the moment and treated accordingly to skip retries,
    // and should stay that way until there's fixes to the exception handling code.
    CLOUD_PROVIDER_ERROR(2),
    UNRECOGNIZED_REQUEST_TYPE(3),
    AUTHENTICATION_ERROR(4),

    GENERAL_ERROR(51),

    CLIENT_GENERAL_ERROR(101),
    CLIENT_CONNECTION_ERROR(102);

    private final int value;

    Code(int value) {
      this.value = value;
    }
  }

  private final Code code;

  public RCSUncheckedException(Code code, String message) {
    super(message);
    this.code = code;
  }

  public RCSUncheckedException(Code code, String message, Throwable cause) {
    super(message, cause);
    this.code = code;
  }

  public RCSUncheckedException(Code code, Throwable cause) {
    super(cause);
    this.code = code;
  }

  public Code getCode() {
    return code;
  }

  @Override
  public String toString() {
    String s = code.name() + ": " + getClass().getName();
    String message = getLocalizedMessage();
    return (message != null) ? (s + ": " + message) : s;
  }
}
