/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.perflogger;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.apache.ranger.raz.s3.lib.RazS3Constants.S3LIB_PERF_TRACKER_ENABLED;

/**
 * Performance logger for Raz S3.
 */
public class PerfLogger {

  private static final Logger LOG = LoggerFactory.getLogger(PerfLogger.class);

  private static PerfLoggerInterface INSTANCE;

  static {
    String enabled = System.getProperty(S3LIB_PERF_TRACKER_ENABLED);
    if (enabled != null && enabled.equalsIgnoreCase("true")) {
      LOG.info("PerfLogger: {}", PerfLoggerImpl.class.getName());
      INSTANCE = PerfLoggerImpl.get();
    } else {
      LOG.info("PerfLogger: {}", PerfLoggerNoOpImpl.class.getName());
      INSTANCE = PerfLoggerNoOpImpl.get();
    }
  }

  // TODO: Add a generic invoker which takes care of setting whether an operation was successful,
  //  along with the generic try-catch-finally block

  public static PerfLoggerInterface get() {
    return INSTANCE;
  }


  public interface PerfLoggerInterface {

    void opBegin(String context, String opName);

    // TODO: Add a flag which indicates whether the operation was successful or not
    long opEnd(String context, String opName);
  }
}