/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.signer.impl;

import org.apache.ranger.raz.s3.lib.HttpMethod;
import org.apache.ranger.raz.s3.lib.signer.SignRequest;

import com.google.common.base.Preconditions;

import java.net.URI;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class SignRequestImpl implements SignRequest {

  // TODO: The following 2 parameters may not be required, and can be hardcoded to s3
  private final String endpointPrefix;
  private final String serviceName;

  private final URI endpoint;
  private final HttpMethod httpMethod;
  private final Map<String, String> headers;
  private final Map<String, List<String>> params;
  private final String resourcePath;
  private final int timeOffset;
  private final byte[] contentToSign;


  public SignRequestImpl(URI endpoint,
      HttpMethod httpMethod, Map<String, String> headers,
      Map<String, List<String>> params, String resourcePath, byte[] contentToSign, int timeOffset,
      String endpointPrefix, String serviceName) {
    Preconditions.checkArgument(endpoint != null, "endpoint must be specified");
    Preconditions.checkArgument(httpMethod != null, "httpMethod must be specified");
    Preconditions.checkArgument(resourcePath != null, "resourcePath must be specified");

    this.endpoint = endpoint;
    this.httpMethod = httpMethod;
    this.headers = headers;
    this.params = params;
    this.resourcePath = resourcePath;
    this.contentToSign = contentToSign;

    this.endpointPrefix = endpointPrefix;
    this.serviceName = serviceName;
    this.timeOffset = timeOffset;
  }

  @Override
  public String getEndpointPrefix() {
    return endpointPrefix;
  }

  @Override
  public String getServiceName() {
    return serviceName;
  }

  @Override
  public URI getEndpoint() {
    return endpoint;
  }

  @Override
  public HttpMethod getHttpMethod() {
    return httpMethod;
  }

  @Override
  public Map<String, String> getHeaders() {
    return headers == null ? Collections.EMPTY_MAP : headers;
  }

  @Override
  public Map<String, List<String>> getParameters() {
    return params == null ? Collections.EMPTY_MAP : params;
  }

  @Override
  public byte[] getContentToSign() {
    return contentToSign;
  }

  @Override
  public String getResourcePath() {
    return resourcePath;
  }

  @Override
  public int getTimeOffset() {
    return timeOffset;
  }

  @Override
  public String toString() {
    return "SignRequestImpl{" +
        "endpointPrefix='" + endpointPrefix + '\'' +
        ", serviceName='" + serviceName + '\'' +
        ", endpoint=" + endpoint +
        ", httpMethod=" + httpMethod +
        ", headers=" + headers +
        ", params=" + params +
        ", resourcePath='" + resourcePath + '\'' +
        ", timeOffset=" + timeOffset + '\'' +
        ", contentToSignExists: " + (contentToSign == null ? false : true) +
        '}';
  }
}