/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.util;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Paths;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

// Private
public class Utils {
  private static final Logger LOG = LoggerFactory.getLogger(Utils.class);

  public static File getFile(String absoluteOrRelativePath)
      throws URISyntaxException {
    LOG.debug("getFile for: {}", absoluteOrRelativePath);
    boolean searchClasspath = false;
    File file = new File(absoluteOrRelativePath);
    if (!file.isAbsolute()) {
      searchClasspath = true;
    }
    if (searchClasspath) {
      URL url = Utils.class.getClassLoader().getResource(absoluteOrRelativePath);
      file = Paths.get(url.toURI()).toFile();
      return file.getAbsoluteFile();
    } else {
      return file;
    }
  }
}
