/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.amazonaws.services.s3;

import com.amazonaws.Request;

/**
 * Not meant for everyday usage.
 *
 * This is part of a series of specialized classes used to extract the {@link
 * com.amazonaws.SignableRequest} generated by the AWS SDK.
 *
 * The underlying AmazonHttpClient is replaced with a custom client, which throws an instance of
 * {@link RequestProvidingException} which contains the {@link com.amazonaws.Request} object
 * generated by the AmazonS3Client.
 *
 * This can be fairly brittle and is dependent on AWS SDK internals.
 */
public class RequestProvidingException extends RuntimeException {

  private final Request<?> request;

  public RequestProvidingException(Request<?> request) {
    super();
    this.request = request;
  }

  public Request<?> getRequest() {
    return request;
  }
}
