/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.s3.lib.aws.parsers;


import org.apache.ranger.raz.s3.lib.aws.parsers.RequestBodyParser.DeleteObjects;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import org.junit.Assert;
import org.junit.Test;

public class TestRequestBodyParsers {

  RequestBodyParser requestBodyParser = new RequestBodyParser();

  @Test(timeout = 20_000L)
  public void testDeleteParsing() {
    String input = null;
    InputStream is;
    DeleteObjects deleteObjects;

    // With and without Version
    input = ("<Delete><Quiet>true</Quiet><Object><Key>test1/</Key><VersionId>1</VersionId></Object><Object><Key>test1/test2/</Key></Object></Delete>");
    is = new ByteArrayInputStream(input.getBytes());
    deleteObjects = requestBodyParser.parseDeleteObjectsRequest(is);
    Assert.assertTrue(deleteObjects.isQuiet());
    Assert.assertEquals(2, deleteObjects.getDeleteEntities().size());
    Assert.assertEquals("test1/", deleteObjects.getDeleteEntities().get(0).getKey());
    Assert.assertEquals("1", deleteObjects.getDeleteEntities().get(0).getVersionId());
    Assert.assertEquals("test1/test2/", deleteObjects.getDeleteEntities().get(1).getKey());
    Assert.assertNull(deleteObjects.getDeleteEntities().get(1).getVersionId());

    // Without quiet
    input = ("<Delete><Object><Key>test1/</Key><VersionId>1</VersionId></Object></Delete>");
    is = new ByteArrayInputStream(input.getBytes());
    deleteObjects = requestBodyParser.parseDeleteObjectsRequest(is);
    Assert.assertFalse(deleteObjects.isQuiet());
    Assert.assertEquals(1, deleteObjects.getDeleteEntities().size());
    Assert.assertEquals("test1/", deleteObjects.getDeleteEntities().get(0).getKey());
    Assert.assertEquals("1", deleteObjects.getDeleteEntities().get(0).getVersionId());

    // XML lower case, quiet upper case
    input = ("<delete><quiet>True</quiet><object><key>test1/</key><versionId>1</versionId></object></delete>");
    is = new ByteArrayInputStream(input.getBytes());
    deleteObjects = requestBodyParser.parseDeleteObjectsRequest(is);
    Assert.assertTrue(deleteObjects.isQuiet());
    Assert.assertEquals(1, deleteObjects.getDeleteEntities().size());
    Assert.assertEquals("test1/", deleteObjects.getDeleteEntities().get(0).getKey());
    Assert.assertEquals("1", deleteObjects.getDeleteEntities().get(0).getVersionId());

    // No Delete Entities
    input = ("<Delete><Quiet>true</Quiet></Delete>");
    is = new ByteArrayInputStream(input.getBytes());
    deleteObjects = requestBodyParser.parseDeleteObjectsRequest(is);
    Assert.assertTrue(deleteObjects.isQuiet());
    Assert.assertEquals(0, deleteObjects.getDeleteEntities().size());
  }


}
