<!---
  Licensed to the Apache Software Foundation (ASF) under one or more
  contributor license agreements.  See the NOTICE file distributed with
  this work for additional information regarding copyright ownership.
  The ASF licenses this file to You under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with
  the License.  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
-->
# How to set up raz in CDP cluster

There are two project to check out and compile: the hadoop and ranger.
Ranger contains the signer processor and shared lib modules.
Hadoop contains the signer client (which will be plugged in to the AWS SDK).

## Starting a cluster
- Use the following jenkins job: https://master-01.jenkins.cloudera.com/job/Cluster-Setup-Cdpd/
- Example with parameters: https://master-01.jenkins.cloudera.com/job/Cluster-Setup-Cdpd/17565/parameters/
- CM and CDH versions can be bquery://product=cm:version=7.2.0:official:L0_PASSED
- Do not use AUTO_TLS (can avoid certificate setup)
- KERBEROS should be KERBEROS(MIT)
- OPTIONAL_ARGS: --include-service-types=ZOOKEEPER,HDFS,YARN,RANGER,KNOX,RANGER_RAZ
- RANGER,KNOX,RANGER_RAZ are should be there in the optional args

## Build ranger raz and copy to the cluster
- Compile Ranger PR with raz s3 first https://github.infra.cloudera.com/CDH/ranger/pull/2
    mvn clean install -DskipTests 
- Compile Hadoop PR with the signer client implementation

- Example script with how to compile copy the raz-s3 processor to the right place:
    https://github.infra.cloudera.com/gabota/custom-scripts/blob/master/raz-s3/razs3_compile.sh
    The script is not yet ready to be used in any environment, but you can get the idea. 
- Example script with how compile and copy the hadoop signer client:
    https://github.infra.cloudera.com/gabota/custom-scripts/blob/master/raz-s3/update_hadoop_aws_on_cluster.sh
    
If you use these script you should change the static vars according to your environment, and parcel version etc...

## Configuration in CM

### Ranger-raz service configuration
In ranger-raz-site.xml add
```xml
<property>
    <name>ranger.raz.processor.s3</name>
    <value>org.apache.ranger.raz.processor.s3.S3RazProcessor</value>
</property>
<property>
  <name>fs.s3a.ext.cab.address</name>
  <value>http://gaborbota-721-1.gaborbota-721.root.hwx.site:8444/gateway</value>
</property>
<property>
  <name>fs.s3a.ext.cab.dt.path</name>
  <value>dt</value>
</property>
<property>
  <name>fs.s3a.ext.cab.path</name>
  <value>aws-cab</value>
</property>
```

### Hadoop (hdfs) service configuration
In core-site.xml add
```xml
<property>
    <name>fs.s3a.custom.signers</name>
    <value>RazS3SignerPlugin:org.apache.ranger.raz.hook.s3.RazS3SignerPlugin:org.apache.ranger.raz.hook.s3.RazS3SignerPluginInitializer</value>
</property>
<property>
    <name>fs.s3a.s3.signing-algorithm</name>
    <value>RazS3SignerPlugin</value>
</property>
<property>
    <name>ranger.raz.client.rest.host.url</name>
    <value>http://gaborbota-1.gaborbota.root.hwx.site:6081</value>
</property>
<property>
  <name>fs.s3a.metadatastore.impl</name>
  <value>org.apache.hadoop.fs.s3a.s3guard.NullMetadataStore</value>
</property>
<property>
  <name>fs.s3a.delegation.token.binding</name>
  <value>org.apache.ranger.raz.hook.s3.RazDelegationTokenBinding</value>
</property>
```

You should restart the services and redeploy configuration.

## Delegation token setup
- Log in on the first node of the cluster with ssh. 
- Switch to the systest user with `su - systest`
- Log in with `kinit` using the password `cloudera`
- Knox binding example: `idbroker.aws.user.role.mapping`: `systest=arn:aws:iam::152813717728:role/gabota-awsint-assumed;rangerraz=arn:aws:iam::152813717728:role/gabota-awsint-assumed`



## [TEMPORARY, WIP] Setup ranger rules
- Script to create the service defs: raz-s3/src/main/resources/service-defs
- Script to create the ranger repo: raz-s3/src/main/resources/create-repo
- Script to create the default ranger policies: raz-s3/src/main/resources/default-ranger-policies