/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.processor.s3;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.ranger.authorization.hadoop.config.RangerPluginConfig;
import org.apache.ranger.plugin.policyengine.RangerAccessResult;
import org.apache.ranger.plugin.service.RangerBasePlugin;

public class RangerS3BasePlugin extends RangerBasePlugin {

  private static final Logger LOG = LoggerFactory.getLogger(RangerS3BasePlugin.class);

  public RangerS3BasePlugin(String serviceType, String appId) {
    super(serviceType, appId);
  }

  public RangerS3BasePlugin(String serviceType, String serviceName, String appId) {
    super(serviceType, serviceName, appId);
  }

  public RangerS3BasePlugin(RangerPluginConfig pluginConfig) {
    super(pluginConfig);
  }

  /**
   * Method to convert NOT_DETERMINED to DENIED for RAZ-S3 calls.
   * @param ret RangerAccessResult to convert.
   */
  @Override
  public void convertAccessResult(RangerAccessResult ret) {
    if (!ret.getIsAccessDetermined()) {
      if(LOG.isDebugEnabled()) {
        LOG.debug("Converting NOT_DETERMINED to DENIED RangerAccessResult...");
      }
      ret.setIsAccessDetermined(true);
      ret.setIsAllowed(false);
    }
  }

}
