/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ranger.raz.processor.s3.signer;

import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.AWSCredentialsProvider;
import org.junit.Test;

import org.apache.hadoop.conf.Configuration;

import static org.apache.hadoop.fs.s3a.Constants.ACCESS_KEY;
import static org.apache.hadoop.fs.s3a.Constants.SECRET_KEY;
import static org.junit.Assert.assertEquals;

public class TestS3RemoteSignerContext {

  @Test
  public void testInitAWSCredentialsChain() {
    final Configuration configuration = new Configuration();
    final String ACCESS_KEY_VAL = "my_access_key";
    final String SECRET_KEY_VAL = "my_secret_key";

    configuration.set(ACCESS_KEY, ACCESS_KEY_VAL);
    configuration.set(SECRET_KEY, SECRET_KEY_VAL);

    S3RemoteSignerContext signerContext = new S3RemoteSignerContext(configuration);
    signerContext.initAWSCredentialsChain();
    final AWSCredentialsProvider awsCredentialProvider
        = signerContext.getAwsCredentialProvider();

    final AWSCredentials credentials = awsCredentialProvider.getCredentials();
    assertEquals("ACCESS_KEY should be set from config.",
        ACCESS_KEY_VAL, credentials.getAWSAccessKeyId());
    assertEquals("SECRET_KEY should be set from config.",
        SECRET_KEY_VAL, credentials.getAWSSecretKey());

  }
}
