/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;


public class RangerRazProcessorClassLoader extends URLClassLoader {
	private static final Logger LOG = LoggerFactory.getLogger(RangerRazProcessorClassLoader.class);

	public RangerRazProcessorClassLoader(String libDir) throws Exception {
		super(RangerRazProcessorClassLoader.getFilesInDirectory(libDir), Thread.currentThread().getContextClassLoader());
    }

    private static URL[] getFilesInDirectory(String dirPath) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("==> RangerRazProcessorClassLoader.getFilesInDirectory({})", dirPath);
        }

        final List<URL> ret = new ArrayList<>();

        try {
            File[] dirFiles = new File(dirPath).listFiles();

            if (dirFiles != null) {
                for(File dirFile : dirFiles) {
                    try {
                        if (!dirFile.canRead()) {
                            LOG.error("RangerRazProcessorClassLoader.getFilesInDirectory({}): {} is not readable!", dirPath,  dirFile.getAbsolutePath());
                        }

                        URL jarPath = dirFile.toURI().toURL();

                        LOG.info("RangerRazProcessorClassLoader.getFilesInDirectory({}): adding {}", dirPath, dirFile.getAbsolutePath());

                        ret.add(jarPath);
                    } catch(Exception excp) {
                        LOG.warn("RangerRazProcessorClassLoader.getFilesInDirectory({}): failed to get URI for file {}", dirPath, dirFile.getAbsolutePath(), excp);
                    }
                }
            }
        } catch(Exception excp) {
            LOG.warn("RangerRazProcessorClassLoader.getFilesInDirectory({}) failed", dirPath, excp);
        }

        if(LOG.isDebugEnabled()) {
            LOG.debug("<== RangerRazProcessorClassLoader.getFilesInDirectory({}): urlCount={}",  dirPath, ret.size());
        }

        return ret.toArray(new URL[ret.size()]);
    }
}
