/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.metrics;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.configuration2.SubsetConfiguration;
import org.apache.hadoop.metrics2.AbstractMetric;
import org.apache.hadoop.metrics2.MetricsRecord;
import org.apache.hadoop.metrics2.MetricsSink;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RazLog4jMetricSink implements MetricsSink{
    private static final Logger              LOG         = LoggerFactory.getLogger(RazLog4jMetricSink.class);
    private static final Logger              RAZ_METRICS = LoggerFactory.getLogger("METRICS");
    private static final Map<String, String> metricNameMap  = new HashMap<>();

    @Override
    public void init(SubsetConfiguration conf) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("===>> RazLog4jMetricSink.init({})", conf);
        }

        metricNameMap.put("AuthzNumOps",  "TotalAuthzRequests");
        metricNameMap.put("AuthzMinTime", "MinTimeTakenToAuthz");
        metricNameMap.put("AuthzMaxTime", "MaxTimeTakenToAuthz");
        metricNameMap.put("AuthzAvgTime", "AvgTimeTakenToAuthz");
        metricNameMap.put("TotalAllowed", "TotalAllowedAuthz");
        metricNameMap.put("TotalDenied",  "TotalDeniedAuthz");
        metricNameMap.put("TotalFailure", "TotalFailureAuthz");
        metricNameMap.put("TotalNotDetermined", "TotalNotDeterminedAuthz");

        if (LOG.isDebugEnabled()) {
            LOG.debug("<<=== RazLog4jMetricSink.init() - metricNameMap={}", metricNameMap);
        }
    }

    @Override
    public void putMetrics(MetricsRecord record) {
        if (LOG.isTraceEnabled()) {
            LOG.trace("===>> RazLog4jMetricSink.putMetrics({})", record);
        }

        String context = record.context();
        if ("auth".equalsIgnoreCase(context)) {
            Map<String, String> metricsMap = new HashMap<>();

            for (AbstractMetric am : record.metrics()) {
                if (metricNameMap.keySet().contains(am.name())) {
                    String value = "";
                    if (am.value() != null) {
                        value = am.value().toString();
                    }
                    metricsMap.put(metricNameMap.get(am.name()), value);
                }
            }
            RAZ_METRICS.info("=== RAZ-AUTHZ-METRIC [{}]", metricsMap);
        }

        if (LOG.isTraceEnabled()) {
            LOG.trace("<<=== RazLog4jMetricSink.putMetrics()");
        }
    }

    @Override
    public void flush() {
        // Implementation not needed
    }

}
