/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.metrics;

import org.apache.hadoop.metrics2.annotation.Metric;
import org.apache.hadoop.metrics2.annotation.Metrics;
import org.apache.hadoop.metrics2.lib.MutableCounterLong;
import org.apache.hadoop.metrics2.lib.MutableRate;
import org.apache.ranger.raz.model.RangerRazResultBase.AccessResult;

@Metrics(context = "auth")
public class RazMetricSource {
	private @Metric(always = true) MutableRate authz;

	private @Metric MutableCounterLong totalAllowed;
	private @Metric MutableCounterLong totalDenied;
	private @Metric MutableCounterLong totalNotDetermined;
	private @Metric MutableCounterLong totalFailure;

	public void update(AccessResult accessResult, long timeConsumed) {
		if (accessResult != null) {
		    authz.add(timeConsumed);

			switch (accessResult) {
			case ALLOWED:
			    totalAllowed.incr();
				break;

			case DENIED:
			    totalDenied.incr();
				break;

			case NOT_DETERMINED:
			    totalNotDetermined.incr();
				break;

			default:
				break;
			}
		} else {
		    totalFailure.incr();
		}
	}
}
