/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.metrics;

import javax.annotation.PostConstruct;

import org.apache.hadoop.metrics2.MetricsSystem;
import org.apache.hadoop.metrics2.lib.DefaultMetricsSystem;
import org.apache.ranger.raz.model.RangerRazResultBase.AccessResult;
import org.springframework.stereotype.Service;

@Service
public class RazMetricsWrapper {
    private final RazMetricSource    source = new RazMetricSource();
    private final RazLog4jMetricSink sink   = new RazLog4jMetricSink();

    @PostConstruct
    public void init() {
        MetricsSystem metricsSystem = DefaultMetricsSystem.initialize("raz");
        sink.init(null);
        metricsSystem.register("source", "Ranger RAZ default metric source", source);
        metricsSystem.register("sink", "Ranger RAZ default metric sink (Log4J)", sink);
    }

    public void update(AccessResult accessResult, long timeConsumed) {
        source.update(accessResult, timeConsumed);
    }
}
