/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.server;


import org.apache.hadoop.conf.Configuration;
import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URL;

public class RangerRazConfig extends Configuration {
	private static final Logger LOG = LoggerFactory.getLogger(RangerRazConfig.class);

	public RangerRazConfig() {
		super(false);

		init();
	}

	private void init() {
		super.addResource("ranger-raz-site.xml");
		copyRazConfigs("core-site.xml");
	}

	private void copyRazConfigs(String filename) {
		Configuration conf = getConfiguration(filename);
		if (conf != null) {
			// copy core-site.xml configurations.
			addResource(conf);
		}
	}

	private Configuration getConfiguration(String filename) {
		Configuration ret = null;
		try {
			URL url = getFileLocation(filename);
			if (url != null) {
				ret = new Configuration(false);
				ret.addResource(url);
			} else {
				LOG.error("Unable to find the file:" + filename);
			}
		} catch (Exception e) {
			LOG.error("Unable to read the file: " + filename, e);
		}
		return ret;
	}

	private URL getFileLocation(String fileName) {
		URL lurl = RangerConfiguration.class.getClassLoader().getResource(fileName);
		if (lurl == null ) {
			lurl = RangerConfiguration.class.getClassLoader().getResource("/" + fileName);
		}
		return lurl;
	}
}
