/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.util;

import org.apache.commons.lang.StringUtils;
import org.apache.hadoop.security.UserGroupInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;


public class RazAuthnUtil {
    private static final Logger LOG = LoggerFactory.getLogger(RazAuthnUtil.class);

    public static String getCurrentUserName() {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        String         ret  = auth != null ? auth.getName() : "";

        if (LOG.isDebugEnabled()) {
            LOG.debug("getCurrentUserName(): ret={}", ret);
        }

        return ret;
    }

    public static Set<String> getCurrentUserGroups() {
        final Set<String> ret;
        String            userName = getCurrentUserName();

        if (StringUtils.isNotEmpty(userName)) {
            UserGroupInformation ugi = UserGroupInformation.createRemoteUser(userName);

            ret = new HashSet<>(ugi.getGroups());
        } else {
            ret = Collections.emptySet();
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("getCurrentUserGroups(): userName={}, ret={}", userName, ret);
        }

        return ret;
    }

    public static String getRequestIpAddress(HttpServletRequest httpServletRequest) {
        String ret = "";

        try {
            InetAddress inetAddr = InetAddress.getByName(httpServletRequest.getRemoteAddr());

            ret = inetAddr.getHostAddress();
        } catch (UnknownHostException ex) {
            LOG.error("Failed to retrieve client IP address", ex);
        }

        return ret;
    }

    public static String[] getForwardedForAddressesFromRequest(HttpServletRequest httpServletRequest){
        String[] ret       = null ;
        String   ipAddress = httpServletRequest.getHeader("X-FORWARDED-FOR");

        if(!StringUtils.isEmpty(ipAddress)){
            ret = ipAddress.split(",");
        }

        return ret;
    }
}
