/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.raz.util;

import java.util.Set;


public class RequestContext {
    private static final ThreadLocal<RequestContext> CURRENT_CONTEXT = new ThreadLocal<>();

    private String      userName;
    private Set<String> userGroups;
    private String      clientIPAddress;
    private String[]    forwardedForAddresses;


    public static RequestContext get() {
        RequestContext ret = CURRENT_CONTEXT.get();

        if (ret == null) {
            ret = new RequestContext();

            CURRENT_CONTEXT.set(ret);
        }

        return ret;
    }

    public static void clear() {
        RequestContext ctx = CURRENT_CONTEXT.get();

        if (ctx != null) {
            ctx.cleanUp();
        }

        CURRENT_CONTEXT.remove();
    }


    private RequestContext() {
    }

    private void cleanUp() {
        // nothing to do for now
    }


    public void setUser(String userName, Set<String> userGroups) {
        this.userName   = userName;
        this.userGroups = userGroups;
    }

    public String getUserName() {
        return userName;
    }

    public Set<String> getUserGroups() {
        return userGroups;
    }

    public String getClientIPAddress() {
        return clientIPAddress;
    }

    public String[] getForwardedForAddresses() {
        return forwardedForAddresses;
    }

    public void setClientIPAddress(String clientIPAddress, String[] forwardedForAddresses) {
        this.clientIPAddress       = clientIPAddress;
        this.forwardedForAddresses = forwardedForAddresses;
    }
}
