/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.rms.server;

import org.apache.log4j.Logger;
import org.apache.ranger.authorization.hadoop.config.RangerConfiguration;
import org.apache.ranger.rms.util.AppConstants;

public class RMSConfig extends RangerConfiguration {

	private static final Logger LOG = Logger.getLogger(RMSConfig.class);
	private static volatile RMSConfig sInstance = null;

	public static RMSConfig getInstance() {
		RMSConfig ret = RMSConfig.sInstance;

		if (ret == null) {
			synchronized (RMSConfig.class) {
				ret = RMSConfig.sInstance;

				if (ret == null) {
					ret = RMSConfig.sInstance = new RMSConfig();
				}
			}
		}

		return ret;
	}

	private RMSConfig() {
		super();
		addRMSResources();
	}

	private boolean addRMSResources() {
		if (LOG.isDebugEnabled()) {
			LOG.debug("==> addRMSResources()");
		}

		boolean ret = true;

		if (!addResourceIfReadable(AppConstants.CONFIG_FILE_PATH)) {
			LOG.error("Could not add " + AppConstants.CONFIG_FILE_PATH + " to RMSConfig.");
			ret = false;
		}

		if (LOG.isDebugEnabled()) {
			LOG.debug("<== addRMSResources(), result=" + ret);
		}

		return ret;
	}
}
