/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

 package org.apache.ranger.rms.view;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyResource;
import org.apache.ranger.plugin.model.RangerServiceResource;

import java.util.Date;
import java.util.Map;

public class RMSNotification implements java.io.Serializable {
	private static final long serialVersionUID = 1L;
	private long notificationId;
	private Date notificationTime;
	private String hmsName;
	private String op;
	private Map<String, RangerPolicyResource> hlResourceElements;
	private Map<String, RangerPolicyResource> llResourceElements;
	private String hlServiceName;
	private String llServiceName;
	private long hlResourceId;
	private long llResourceId;

	private RangerServiceResource hlServiceResource;
	private RangerServiceResource llServiceResource;
	private String                ownerName;
	private Map<String, String>   additionalInfo;

	public RMSNotification(long notificationId, Date notificationTime, String hmsName, String op, Map<String, RangerPolicyResource> hlResourceElements, Map<String, RangerPolicyResource> llResourceElements) {
		this(notificationId, notificationTime, hmsName, op, hlResourceElements, llResourceElements,null);
	}

	public RMSNotification(long notificationId, Date notificationTime, String hmsName, String op, Map<String, RangerPolicyResource> hlResourceElements, Map<String, RangerPolicyResource> llResourceElements, Map<String, String> additionalInfo) {
		this(notificationId, notificationTime, hmsName, op, hlResourceElements, llResourceElements, -1L, null, -1L, null, null, additionalInfo);
	}

	public RMSNotification(long notificationId, Date notificationTime, String hmsName, String op, Map<String, RangerPolicyResource> hlResourceElements, Map<String, RangerPolicyResource> llResourceElements, String ownerName, Map<String, String> additionalInfo) {
		this(notificationId, notificationTime, hmsName, op, hlResourceElements, llResourceElements, -1L, null, -1L, null, ownerName, additionalInfo);
	}


	public RMSNotification(RMSNotification other) {
		this(other.notificationId, other.notificationTime, other.hmsName, other.op, other.hlResourceElements, other.llResourceElements, other.hlResourceId, other.hlServiceName, other.llResourceId, other.llServiceName, other.ownerName, other.additionalInfo);
	}

	public RMSNotification(long notificationId, Date notificationTime, String hmsName, String op, Map<String, RangerPolicyResource> hlResourceElements, Map<String, RangerPolicyResource> llResourceElements, long hlResourceId, String hlServiceName, long llResourceId, String llServiceName, String ownerName, Map<String, String> additionalInfo) {
		setNotificationID(notificationId);
		setNotificationTime(notificationTime);
		setHmsName(hmsName);
		setOp(op);
		setHlResource(hlResourceElements);
		setLlResource(llResourceElements);
		setHlResourceId(hlResourceId);
		setHlServiceName(hlServiceName);
		setLlResourceId(llResourceId);
		setLlServiceName(llServiceName);
		setOwnerName(ownerName);
		setAdditionalInfo(additionalInfo);
	}

	public String getOp() {
		return op;
	}

	public void setOp(String op) {
		this.op = op;
	}

	public long getNotificationID() {
		return notificationId;
	}

	public void setNotificationID(long notificationId) {
		this.notificationId = notificationId;
	}

	public Date getNotificationTime() {
		return notificationTime;
	}

	public void setNotificationTime(Date notificationTime) {
		this.notificationTime = notificationTime;
	}

	public String getHmsName() { return hmsName; }

	public void setHmsName(String hmsName) { this.hmsName = hmsName; }
	
	public void setHlResource(Map<String, RangerPolicyResource> hlResourceElements) { this.hlResourceElements = hlResourceElements; this.hlServiceResource = null; }

	public Map<String, RangerPolicyResource> getHlResource() { return hlResourceElements; }

	public void setLlResource(Map<String, RangerPolicyResource> llResourceElements) { this.llResourceElements = llResourceElements; this.llServiceResource = null; }
	
	public Map<String, RangerPolicyResource> getLlResource() { return llResourceElements; }

	public long getHlResourceId() { return hlResourceId; }

	public void setHlResourceId(long hlResourceId) { this.hlResourceId = hlResourceId; }

	public long getLlResourceId() { return llResourceId; }

	public void setLlResourceId(long llResourceId) { this.llResourceId = llResourceId; }

	public String getHlServiceName() { return hlServiceName; }

	public void setHlServiceName(String hlServiceName) { this.hlServiceName = hlServiceName; this.hlServiceResource = null; }

	public String getLlServiceName() { return llServiceName; }

	public void setLlServiceName(String llServiceName) { this.llServiceName = llServiceName; this.llServiceResource = null; }

	public RangerServiceResource getHlAsServiceResource() {
		RangerServiceResource ret = hlServiceResource;
		if (ret == null) {
			if (StringUtils.isNotBlank(hlServiceName) && hlResourceElements != null) {
				hlServiceResource = new RangerServiceResource("", hlServiceName, hlResourceElements);
				ret = hlServiceResource;
			}
		}
		return ret;
	}

	public RangerServiceResource getLlAsServiceResource() {
		RangerServiceResource ret = llServiceResource;
		if (ret == null) {
			if (StringUtils.isNotBlank(llServiceName) && llResourceElements != null) {
				llServiceResource = new RangerServiceResource("", llServiceName, llResourceElements);
				ret = llServiceResource;
			}
		}
		return ret;
	}

	public String getOwnerName() {
		return ownerName;
	}

	public void setOwnerName(String ownerName) {
		this.ownerName = ownerName;
	}

	public Map<String, String> getAdditionalInfo() {
		return additionalInfo;
	}

	public void setAdditionalInfo(Map<String, String> additionalInfo) {
		this.additionalInfo = additionalInfo;
	}

	public String toString( ) {
		String str = "RMSNotification={";

		str += "notificationId={" + notificationId + "} ";
		str += "notificationTime={" + notificationTime + "} ";
		str += "hmsName={" + hmsName + "} ";
		str += "op={" + op + "} ";
		str += "hlResourceElements={" + hlResourceElements + "} ";
		str += "llResourceElements={" + llResourceElements + "} ";
		str += "hlServiceName={" + hlServiceName + "} ";
		str += "hlResourceId={" + hlResourceId + "} ";
		str += "llServiceName={" + llServiceName + "} ";
		str += "llResourceId={" + llResourceId + "} ";
		str += "ownerName={" + ownerName + "} ";
		str += "additionalInfo{";
		if (MapUtils.isNotEmpty(additionalInfo)) {
			for (Map.Entry<String, String> entry : additionalInfo.entrySet()) {
				str += entry.getKey() + "=" + entry.getValue() + ", ";
			}
		}
		str += "}";

		str += "}";

		return str;
	}
}
