/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.hms;

import org.apache.hadoop.hive.metastore.HiveMetaStoreClient;

import com.google.common.base.Preconditions;

public class HMSClient implements AutoCloseable {
	private final HiveMetaStoreClient client;

	private boolean valid;
	private String  hmsName;

	public HMSClient(HiveMetaStoreClient client, String hmsName) {
		this.client = Preconditions.checkNotNull(client);
		this.valid = true;
		this.hmsName = hmsName;
	}

	public void setHmsName(String hmsName) { this.hmsName = hmsName; }

	public String getHmsName() { return hmsName; }

	public HiveMetaStoreClient getClient() {
		return this.client;
	}

	public void invalidate() {
		if (this.valid) {
			this.client.close();
			this.valid = false;
		}
	}

	public void close() {
		if (this.valid) {
			this.client.close();
			this.valid = false;
		}
	}
}
