/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.ranger.plugin.util;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.ObjectUtils;
import org.apache.ranger.plugin.policyengine.RangerAccessResourceImpl;

import java.util.HashMap;
import java.util.Map;

public class RangerRMSAccessResourceImpl extends RangerAccessResourceImpl {
    private Map<String, String> additionalInfo;

    public RangerRMSAccessResourceImpl() {
        this(null);
    }
    public RangerRMSAccessResourceImpl(Map<String, String> additionalInfo) {
        super();
        setAdditionalInfo(additionalInfo);
    }

    public Map<String, String> getAdditionalInfo() {
        return additionalInfo;
    }

    public void setAdditionalInfo(Map<String, String> additionalInfo) {
        this.additionalInfo = additionalInfo == null ? new HashMap<>() : additionalInfo;
    }

    @Override
    public boolean equals(Object obj) {
        if (!super.equals(obj)) {
            return false;
        }

        if(obj == null || !(obj instanceof RangerRMSAccessResourceImpl)) {
            return false;
        }

        RangerRMSAccessResourceImpl other = (RangerRMSAccessResourceImpl) obj;

        return ObjectUtils.equals(additionalInfo, other.additionalInfo);
    }

    @Override
    public int hashCode() {
        int ret = super.hashCode();

        if (MapUtils.isNotEmpty(additionalInfo)) {
            ret = 31 * ObjectUtils.hashCode(additionalInfo);
        }

        return ret;
    }

    @Override
    public String toString() {
        String ret = super.toString();
        if (MapUtils.isNotEmpty(additionalInfo)) {
            StringBuilder sb = new StringBuilder();
            sb.append("{");
            for (Map.Entry<String, String> entry : additionalInfo.entrySet()) {
                sb.append("{").append(entry.getKey()).append("=").append(entry.getValue()).append("},");
            }
            sb.append("}");
            ret += sb.toString();
        }
        return ret;
    }
}
