#!/bin/bash
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#	 http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------------
#
# ranger-rms Setup Script
#
# This script will install ranger-rms webapplication under tomcat and also, initialize the database with ranger-rms users/tables.

PROPFILE=$PWD/install.properties
propertyValue=''

if [ ! -f ${PROPFILE} ]
then
	echo "$PROPFILE file not found....!!";
	exit 1;
fi

usage() {
  [ "$*" ] && echo "$0: $*"
  sed -n '/^##/,/^$/s/^## \{0,1\}//p' "$0"
  exit 2
} 2>/dev/null

log() {
   local prefix="$(date +%Y-%m-%d\ %H:%M:%S,%3N) "
   echo "${prefix} $@" >> $LOGFILE
   echo "${prefix} $@"
}

get_prop(){
	validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
	if  test -z "$validateProperty" ; then log "[E] '$1' not found in $2 file while getting....!!"; exit 1; fi
	value=$(echo $validateProperty | cut -d "=" -f2-)
	echo $value
}

PYTHON_COMMAND_INVOKER=$(get_prop 'PYTHON_COMMAND_INVOKER' $PROPFILE)
DB_FLAVOR=$(get_prop 'DB_FLAVOR' $PROPFILE)
SQL_CONNECTOR_JAR=$(get_prop 'SQL_CONNECTOR_JAR' $PROPFILE)
db_root_user=$(get_prop 'db_root_user' $PROPFILE)
db_root_password=$(get_prop 'db_root_password' $PROPFILE)
db_host=$(get_prop 'db_host' $PROPFILE)
db_name=$(get_prop 'db_name' $PROPFILE)
db_user=$(get_prop 'db_user' $PROPFILE)
db_password=$(get_prop 'db_password' $PROPFILE)
db_ssl_enabled=$(get_prop 'db_ssl_enabled' $PROPFILE)
db_ssl_required=$(get_prop 'db_ssl_required' $PROPFILE)
db_ssl_verifyServerCertificate=$(get_prop 'db_ssl_verifyServerCertificate' $PROPFILE)
db_ssl_auth_type=$(get_prop 'db_ssl_auth_type' $PROPFILE)
unix_user=$(get_prop 'unix_user' $PROPFILE)
unix_user_pwd=$(get_prop 'unix_user_pwd' $PROPFILE)
unix_group=$(get_prop 'unix_group' $PROPFILE)
POLICY_MGR_URL=$(get_prop 'POLICY_MGR_URL' $PROPFILE)
POLICY_MGR_SSL_CONFIG_FILE=$(get_prop 'POLICY_MGR_SSL_CONFIG_FILE' $PROPFILE)
RMS_DIR=$(eval echo "$(get_prop 'RMS_DIR' $PROPFILE)")
app_home=$(eval echo "$(get_prop 'app_home' $PROPFILE)")
TMPFILE=$(eval echo "$(get_prop 'TMPFILE' $PROPFILE)")
LOGFILE=$(eval echo "$(get_prop 'LOGFILE' $PROPFILE)")
JAVA_BIN=$(get_prop 'JAVA_BIN' $PROPFILE)
JAVA_VERSION_REQUIRED=$(get_prop 'JAVA_VERSION_REQUIRED' $PROPFILE)
JAVA_ORACLE=$(get_prop 'JAVA_ORACLE' $PROPFILE)
mysql_core_file=$(get_prop 'mysql_core_file' $PROPFILE)
oracle_core_file=$(get_prop 'oracle_core_file' $PROPFILE)
postgres_core_file=$(get_prop 'postgres_core_file' $PROPFILE)
sqlserver_core_file=$(get_prop 'sqlserver_core_file' $PROPFILE)
sqlanywhere_core_file=$(get_prop 'sqlanywhere_core_file' $PROPFILE)
cred_keystore_filename=$(eval echo "$(get_prop 'cred_keystore_filename' $PROPFILE)")
RMS_LOG_DIR=$(eval echo "$(get_prop 'RMS_LOG_DIR' $PROPFILE)")
RMS_PID_DIR_PATH=$(eval echo "$(get_prop 'RMS_PID_DIR_PATH' $PROPFILE)")
hive_site_file_path=$(eval echo "$(get_prop 'hive_site_file_path' $PROPFILE)")

host_name=$(get_prop 'host_name' $PROPFILE)
authentication_mode=$(get_prop 'authentication_mode' $PROPFILE)
ranger_rms_principal=$(get_prop 'ranger_rms_principal' $PROPFILE)
ranger_rms_keytab=$(get_prop 'ranger_rms_keytab' $PROPFILE)
hadoop_conf=$(get_prop 'hadoop_conf' $PROPFILE)

DB_HOST="${db_host}"

ranger_rms_http_enabled=$(get_prop 'ranger_rms_http_enabled' $PROPFILE)
ranger_rms_https_keystore_file=$(get_prop 'ranger_rms_https_keystore_file' $PROPFILE)
ranger_rms_https_keystore_keyalias=$(get_prop 'ranger_rms_https_keystore_keyalias' $PROPFILE)
ranger_rms_https_keystore_password=$(get_prop 'ranger_rms_https_keystore_password' $PROPFILE)

javax_net_ssl_keyStore=$(get_prop 'javax_net_ssl_keyStore' $PROPFILE)
javax_net_ssl_keyStorePassword=$(get_prop 'javax_net_ssl_keyStorePassword' $PROPFILE)
javax_net_ssl_trustStore=$(get_prop 'javax_net_ssl_trustStore' $PROPFILE)
javax_net_ssl_trustStorePassword=$(get_prop 'javax_net_ssl_trustStorePassword' $PROPFILE)
# HA related property
ranger_rms_server_ha_enabled=$(get_prop 'ranger_rms_server_ha_enabled' $PROPFILE)
ranger_rms_server_ha_zookeeper_zkroot=$(get_prop 'ranger_rms_server_ha_zookeeper_zkroot' $PROPFILE)
ranger_rms_server_ha_zookeeper_connect=$(get_prop 'ranger_rms_server_ha_zookeeper_connect' $PROPFILE)
ranger_rms_server_ids=$(get_prop 'ranger_rms_server_ids' $PROPFILE)
ranger_rms_server_address_id1=$(get_prop 'ranger_rms_server_address_id1' $PROPFILE)
ranger_rms_server_address_id2=$(get_prop 'ranger_rms_server_address_id2' $PROPFILE)
ranger_rms_server_address_id3=$(get_prop 'ranger_rms_server_address_id3' $PROPFILE)
ranger_rms_server_ha_zookeeper_retry_sleeptime_ms=$(get_prop 'ranger_rms_server_ha_zookeeper_retry_sleeptime_ms' $PROPFILE)
ranger_rms_server_ha_zookeeper_session_timeout_ms=$(get_prop 'ranger_rms_server_ha_zookeeper_session_timeout_ms' $PROPFILE)
ranger_rms_server_ha_zookeeper_num_retries=$(get_prop 'ranger_rms_server_ha_zookeeper_num_retries' $PROPFILE)
ranger_rms_server_ha_zookeeper_acl=$(get_prop 'ranger_rms_server_ha_zookeeper_acl' $PROPFILE)
ranger_rms_server_ha_zookeeper_acl_auth=$(get_prop 'ranger_rms_server_ha_zookeeper_acl_auth' $PROPFILE)

hive_metastore_uris=$(get_prop 'hive_metastore_uris' $PROPFILE)
hive_metastore_sasl_enabled=$(get_prop 'hive_metastore_sasl_enabled' $PROPFILE)
hive_metastore_client_socket_timeout=$(get_prop 'hive_metastore_client_socket_timeout' $PROPFILE)
hive_metastore_service_host=$(get_prop 'hive_metastore_service_host' $PROPFILE)
ranger_rms_HMS_map_managed_tables=$(get_prop 'ranger_rms_HMS_map_managed_tables' $PROPFILE)
ranger_rms_HMS_source_service_name=$(get_prop 'ranger_rms_HMS_source_service_name' $PROPFILE)
ranger_rms_HMS_target_service_name=$(get_prop 'ranger_rms_HMS_target_service_name' $PROPFILE)
ranger_rms_polling_notifications_frequency_ms=$(get_prop 'ranger_rms_polling_notifications_frequency_ms' $PROPFILE)
ranger_rms_transaction_size_for_persisting_mappings=$(get_prop 'ranger_rms_transaction_size_for_persisting_mappings' $PROPFILE)

check_ret_status(){
	if [ $1 -ne 0 ]; then
		log "[E] $2";
		exit 1;
	fi
}

check_ret_status_for_groupadd(){
# 9 is the response if the group exists
	if [ $1 -ne 0 ] && [ $1 -ne 9 ]; then
		log "[E] $2";
		exit 1;
	fi
}

is_command () {
	log "[I] check if command $1 exists"
	type "$1" >/dev/null
}

get_distro(){
	log "[I] Checking distribution name.."
	ver=$(cat /etc/*{issues,release,version} 2> /dev/null)
	if [[ $(echo $ver | grep DISTRIB_ID) ]]; then
		DIST_NAME=$(lsb_release -si)
	else
		DIST_NAME=$(echo $ver | cut -d ' ' -f 1 | sort -u | head -1)
	fi
	export $DIST_NAME
	log "[I] Found distribution : $DIST_NAME"

}
#Get Properties from File without erroring out if property is not there
#$1 -> propertyName $2 -> fileName $3 -> variableName $4 -> failIfNotFound
getPropertyFromFileNoExit(){
	validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
	if  test -z "$validateProperty" ; then 
		log "[E] '$1' not found in $2 file while getting....!!";
		if [ $4 == "true" ] ; then
			exit 1;
		else
			value=""
		fi
	else
		value=$(echo $validateProperty | cut -d "=" -f2-)
	fi
	eval $3="'$value'"
}
#Get Properties from File
#$1 -> propertyName $2 -> fileName $3 -> variableName
getPropertyFromFile(){
	validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
	if  test -z "$validateProperty" ; then log "[E] '$1' not found in $2 file while getting....!!"; exit 1; fi
	value=$(echo $validateProperty | cut -d "=" -f2-)
	eval $3="'$value'"
}

#Update Properties to File
#$1 -> propertyName $2 -> newPropertyValue $3 -> fileName
updatePropertyToFile(){
	sed -i 's@^'$1'=[^ ]*$@'$1'='$2'@g' $3
	#validate=`sed -i 's/^'$1'=[^ ]*$/'$1'='$2'/g' $3`	#for validation
	validate=$(sed '/^\#/d' $3 | grep "^$1"  | tail -n 1 | cut -d "=" -f2-) # for validation
	#echo 'V1:'$validate
	if test -z "$validate" ; then log "[E] '$1' not found in $3 file while Updating....!!"; exit 1; fi
	log "[I] File $3 Updated successfully : {'$1'}"
}

#Update Properties to File
#$1 -> propertyName $2 -> newPropertyValue $3 -> fileName
updatePropertyToFilePy(){
	$PYTHON_COMMAND_INVOKER update_property.py $1 $2 $3
	check_ret_status $? "Update property failed for: {'$1'}"
}

check_user_pwd(){
	if [ -z "$1" ]; then
		log "[E] The unix user password is empty. Please set user password.";
		exit 1;
	fi
}

password_validation(){
	if [ -z "$1" ]
	then
		log "[I] Blank password is not allowed for" $2". Please enter valid password."
		exit 1
	else
		if [[ $1 =~ [\"\'\`\\\] ]]
		then
			log "[E]" $2 "password contains one of the unsupported special characters:\" ' \` \\"
			exit 1
		else
			log "[I]" $2 "password validated."
		fi
	fi
}


init_variables(){
	curDt=`date '+%Y%m%d%H%M%S'`

	if [ -f ${PWD}/version ] 
	then
		VERSION=`cat ${PWD}/version`
	else
		VERSION="0.5.0"
	fi

	RMS_DIR=$PWD

	ranger_rms=ranger-rms
	ranger_rms_initd=ranger-rms-initd
	INSTALL_DIR=${RMS_DIR}

	WEBAPP_ROOT=${INSTALL_DIR}/ews/webapp

	DB_FLAVOR=`echo $DB_FLAVOR | tr '[:lower:]' '[:upper:]'`
	if [ "${DB_FLAVOR}" == "" ]
	then
		DB_FLAVOR="MYSQL"
	fi
	log "[I] DB_FLAVOR=${DB_FLAVOR}"

	db_ssl_enabled=`echo $db_ssl_enabled | tr '[:upper:]' '[:lower:]'`
	if [ "${db_ssl_enabled}" != "true" ]
	then
		db_ssl_enabled="false"
		db_ssl_required="false"
		db_ssl_verifyServerCertificate="false"
		db_ssl_auth_type="2-way"
	fi
	if [ "${db_ssl_enabled}" == "true" ]
	then
		db_ssl_required=`echo $db_ssl_required | tr '[:upper:]' '[:lower:]'`
		db_ssl_verifyServerCertificate=`echo $db_ssl_verifyServerCertificate | tr '[:upper:]' '[:lower:]'`
		db_ssl_auth_type=`echo $db_ssl_auth_type | tr '[:upper:]' '[:lower:]'`
		if [ "${db_ssl_required}" != "true" ]
		then
			db_ssl_required="false"
		fi
		if [ "${db_ssl_verifyServerCertificate}" != "true" ]
		then
			db_ssl_verifyServerCertificate="false"
		fi
		if [ "${db_ssl_auth_type}" != "1-way" ]
		then
			db_ssl_auth_type="2-way"
		fi
	fi
}


check_python_command() {
		if is_command ${PYTHON_COMMAND_INVOKER} ; then
			log "[I] '${PYTHON_COMMAND_INVOKER}' command found"
		else
			log "[E] '${PYTHON_COMMAND_INVOKER}' command not found"
			exit 1;
		fi
}

run_dba_steps(){
	getPropertyFromFileNoExit 'setup_mode' $PROPFILE setup_mode false
	if [ "x${setup_mode}x" == "xSeparateDBAx" ]; then
		log "[I] Setup mode is set to SeparateDBA. Not Running DBA steps. Please run dba_script.py before running setup..!";
	else
		log "[I] Setup mode is not set. Running DBA steps..";
		$PYTHON_COMMAND_INVOKER dba_script.py -q
	fi
}
check_db_connector() {
	log "[I] Checking ${DB_FLAVOR} CONNECTOR FILE : ${SQL_CONNECTOR_JAR}"
	if test -f "$SQL_CONNECTOR_JAR"; then
		log "[I] ${DB_FLAVOR} CONNECTOR FILE : $SQL_CONNECTOR_JAR file found"
	else
		log "[E] ${DB_FLAVOR} CONNECTOR FILE : $SQL_CONNECTOR_JAR does not exists" ; exit 1;
	fi
}

check_java_version() {
	#Check for JAVA_HOME
	if [ "${JAVA_HOME}" == "" ]
	then
		log "[E] JAVA_HOME environment property not defined, aborting installation."
		exit 1
	fi

	export JAVA_BIN=${JAVA_HOME}/bin/java

	if is_command ${JAVA_BIN} ; then
		log "[I] '${JAVA_BIN}' command found"
	else
		log "[E] '${JAVA_BIN}' command not found"
		exit 1;
	fi

	version=$("$JAVA_BIN" -version 2>&1 | awk -F '"' '/version/ {print $2}')
	major=`echo ${version} | cut -d. -f1`
	minor=`echo ${version} | cut -d. -f2`
	current_java_version="$major.$minor"
	num_current_java_version=`echo $current_java_version|awk ' { printf("%3.2f\n", $0); } '`
	JAVA_VERSION_REQUIRED=`echo $JAVA_VERSION_REQUIRED | awk '{gsub(/ /,"")}1'`
	JAVA_VERSION_REQUIRED=`echo $JAVA_VERSION_REQUIRED | awk '{gsub(/'"'"'/,"")}1'`
	num_required_java_version=`echo $JAVA_VERSION_REQUIRED|awk ' { printf("%3.2f\n", $0); } '`
	if [ `echo "$num_current_java_version < $num_required_java_version" | bc` -eq 1 ];then
		log "[E] The java version must be greater than or equal to $JAVA_VERSION_REQUIRED, the current java version is $version"
		exit 1;
	fi
}

sanity_check_files() {
	if test -d $app_home; then
		log "[I] $app_home folder found"
	else
		log "[E] $app_home does not exists" ; exit 1;
	fi
	if [ "${DB_FLAVOR}" == "MYSQL" ]
	then
		if test -f $mysql_core_file; then
			log "[I] $mysql_core_file file found"
		else
			log "[E] $mysql_core_file does not exists" ; exit 1;
		fi
	fi
	if [ "${DB_FLAVOR}" == "ORACLE" ]
	then
		if test -f ${oracle_core_file}; then
			log "[I] ${oracle_core_file} file found"
		else
			log "[E] ${oracle_core_file} does not exists" ; exit 1;
		fi
	fi
	if [ "${DB_FLAVOR}" == "POSTGRES" ]
	then
		if test -f ${postgres_core_file}; then
			log "[I] ${postgres_core_file} file found"
		else
			log "[E] ${postgres_core_file} does not exists" ; exit 1;
		fi
	fi
	if [ "${DB_FLAVOR}" == "MSSQL" ]
	then
		if test -f ${sqlserver_core_file}; then
			log "[I] ${sqlserver_core_file} file found"
		else
			log "[E] ${sqlserver_core_file} does not exists" ; exit 1;
		fi
	fi
	if [ "${DB_FLAVOR}" == "SQLA" ]
	then
		if [ "${LD_LIBRARY_PATH}" == "" ]
		then
			log "[E] LD_LIBRARY_PATH environment property not defined, aborting installation."
			exit 1
		fi
		if test -f ${sqlanywhere_core_file}; then
			log "[I] ${sqlanywhere_core_file} file found"
		else
			log "[E] ${sqlanywhere_core_file} does not exists" ; exit 1;
		fi
	fi
}

create_rollback_point() {
	DATE=`date`
	BAK_FILE=$APP-$VERSION.$DATE.bak
	log "Creating backup file : $BAK_FILE"
	cp "$APP" "$BAK_FILE"
}

copy_db_connector(){
	libfolder=$PWD/ews/lib
	if [ ! -d  ${libfolder} ]
	then
		log "Creating ${libfolder}"
		mkdir -p ${libfolder}
	fi
	fn=`basename ${SQL_CONNECTOR_JAR}`
	if [ ! -f ${libfolder}/${fn} ]
	then
		log "[I] Copying ${DB_FLAVOR} Connector to ${libfolder} ";
		cp -f $SQL_CONNECTOR_JAR ${libfolder}
		check_ret_status $? "Copying ${DB_FLAVOR} Connector to ${libfolder} failed"
		log "[I] Copying ${DB_FLAVOR} Connector to ${libfolder} DONE";
	else
		log "[I] Using already existing DB connector: ${libfolder}/${fn} ";
	fi
}

update_properties() {
	newPropertyValue=''
	echo "export JAVA_HOME=${JAVA_HOME}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/java_home.sh
	chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/java_home.sh

	to_file=$PWD/ews/webapp/WEB-INF/classes/conf/ranger-rms-site.xml
	if test -f $to_file; then
		log "[I] $to_file file found"
	else
		log "[E] $to_file does not exists" ; exit 1;
	fi

	if [ "${POLICY_MGR_URL}" != "" ]
	then
		propertyName=ranger.admin.rest.url
		newPropertyValue="${POLICY_MGR_URL}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi
	if [ "${POLICY_MGR_SSL_CONFIG_FILE}" != "" ]
	then
		propertyName=ranger.admin.rest.ssl.config.file
		newPropertyValue="${POLICY_MGR_SSL_CONFIG_FILE}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi
	if [ "${db_ssl_enabled}" != "" ]
	then
		propertyName=ranger-rms.db.ssl.enabled
		newPropertyValue="${db_ssl_enabled}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.db.ssl.required
		newPropertyValue="${db_ssl_required}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.db.ssl.verifyServerCertificate
		newPropertyValue="${db_ssl_verifyServerCertificate}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.db.ssl.auth.type
		newPropertyValue="${db_ssl_auth_type}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi

	propertyName=ranger-rms.db.flavor
	newPropertyValue=`echo $DB_FLAVOR | tr '[:upper:]' '[:lower:]'`
	updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	if [ "${DB_FLAVOR}" == "MYSQL" ]
	then
		propertyName=ranger-rms.jpa.jdbc.url
		newPropertyValue="jdbc:log4jdbc:mysql://${DB_HOST}/${db_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.dialect
		newPropertyValue="org.eclipse.persistence.platform.database.MySQLPlatform"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.driver
		newPropertyValue="net.sf.log4jdbc.DriverSpy"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi
	if [ "${DB_FLAVOR}" == "ORACLE" ]
	then
		propertyName=ranger-rms.jpa.jdbc.url
		count=$(grep -o ":" <<< "$DB_HOST" | wc -l)
		#if [[ ${count} -eq 2 ]] ; then
		if [ ${count} -eq 2 ] || [ ${count} -eq 0 ]; then
			#jdbc:oracle:thin:@[HOST][:PORT]:SID or #jdbc:oracle:thin:@GL
			newPropertyValue="jdbc:oracle:thin:@${DB_HOST}"
		else
			#jdbc:oracle:thin:@//[HOST][:PORT]/SERVICE
			newPropertyValue="jdbc:oracle:thin:@//${DB_HOST}"
		fi
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.dialect
		newPropertyValue="org.eclipse.persistence.platform.database.OraclePlatform"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.driver
		newPropertyValue="oracle.jdbc.OracleDriver"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi
	if [ "${DB_FLAVOR}" == "POSTGRES" ]
	then
		propertyName=ranger-rms.jpa.jdbc.url
		newPropertyValue="jdbc:postgresql://${DB_HOST}/${db_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.dialect
		newPropertyValue="org.eclipse.persistence.platform.database.PostgreSQLPlatform"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.driver
		newPropertyValue="org.postgresql.Driver"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi
	if [ "${DB_FLAVOR}" == "MSSQL" ]
	then
		propertyName=ranger-rms.jpa.jdbc.url
		newPropertyValue="jdbc:sqlserver://${DB_HOST};databaseName=${db_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.dialect
		newPropertyValue="org.eclipse.persistence.platform.database.SQLServerPlatform"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.driver
		newPropertyValue="com.microsoft.sqlserver.jdbc.SQLServerDriver"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi
	if [ "${DB_FLAVOR}" == "SQLA" ]
	then
		propertyName=ranger-rms.jpa.jdbc.url
		newPropertyValue="jdbc:sqlanywhere:database=${db_name};host=${DB_HOST}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.dialect
		newPropertyValue="org.eclipse.persistence.platform.database.SQLAnywherePlatform"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.driver
		newPropertyValue="sap.jdbc4.sqlanywhere.IDriver"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi
	propertyName=ranger-rms.jpa.jdbc.user
	newPropertyValue="${db_user}"
	updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	keystore="${cred_keystore_filename}"

	echo "Starting configuration for ranger-rms DB credentials:"

	DB_CREDENTIAL_ATTR="ranger-rms.jpa.jdbc.password"
	DB_CREDENTIAL_ALIAS="ranger-rms.jpa.jdbc.credential.alias"

	if [ "${keystore}" != "" ]
	then
		mkdir -p `dirname "${keystore}"`
		$PYTHON_COMMAND_INVOKER ranger_credential_helper.py -l "cred/lib/*" -f "$keystore" -k "$DB_CREDENTIAL_ALIAS" -v "$db_password" -c 1

		propertyName=ranger-rms.jpa.jdbc.credential.alias
		newPropertyValue="${DB_CREDENTIAL_ALIAS}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.credential.provider.path
		newPropertyValue="${keystore}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=ranger-rms.jpa.jdbc.password
		newPropertyValue="_"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	else
		propertyName="${DB_CREDENTIAL_ATTR}"
		newPropertyValue="${db_password}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi

	if test -f $keystore; then
		#echo "$keystore found."
		chown -R ${unix_user}:${unix_group} ${keystore}
		chmod 640 ${keystore}
	fi

	########### KERBEROS CONFIG ############
	if [ "${host_name}" != "" ]
	then
		propertyName=ranger-rms.service.host
		newPropertyValue="${host_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	else
		propertyName=ranger-rms.service.host
		newPropertyValue=`hostname`
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi

	if [ "${authentication_mode}" == "kerberos" ]
	then
		propertyName=hadoop.security.authentication
		newPropertyValue="${authentication_mode}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	else
		propertyName=hadoop.security.authentication
		newPropertyValue="simple"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi

	if [ "${ranger_rms_principal}" != "" ]
	then
		propertyName=ranger-rms.kerberos.principal
		newPropertyValue="${ranger_rms_principal}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=HiveMetaStore.kerberos.principal
		newPropertyValue="${ranger_rms_principal}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

	fi

	if [ "${ranger_rms_keytab}" != "" ]
	then
		propertyName=ranger-rms.kerberos.keytab
		newPropertyValue="${ranger_rms_keytab}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file

		propertyName=HiveMetaStore.kerberos.keytab
		newPropertyValue="${ranger_rms_keytab}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file
	fi

	to_file_ranger_rms_site=$PWD/ews/webapp/WEB-INF/classes/conf/ranger-rms-site.xml
	if test -f $to_file_ranger_rms_site; then
		log "[I] $to_file_ranger_rms_site file found"
	else
		log "[E] $to_file_ranger_rms_site does not exists" ; exit 1;
	fi

	propertyName=ranger-rms.service.http.enabled
	newPropertyValue="${ranger_rms_http_enabled}"
	updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	if [ "${ranger_rms_http_enabled}" == "false" ]
	then
		if [ "${ranger_rms_https_keystore_keyalias}" == "" ]
		then
			ranger_rms_https_keystore_keyalias=ranger-rms
		fi
		if [ "${ranger_rms_https_keystore_file}" != "" ] && [ "${ranger_rms_https_keystore_password}" != "" ]
		then
			propertyName=ranger-rms.service.https.attrib.ssl.enabled
			newPropertyValue="true"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			propertyName=ranger-rms.service.https.attrib.client.auth
			newPropertyValue="want"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			propertyName=ranger-rms.service.https.attrib.keystore.file
			newPropertyValue="${ranger_rms_https_keystore_file}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			propertyName=ranger-rms.service.https.attrib.keystore.keyalias
			newPropertyValue="${ranger_rms_https_keystore_keyalias}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			policymgr_https_keystore_credential_alias=keyStoreCredentialAlias
			propertyName=ranger-rms.service.https.attrib.keystore.credential.alias
			newPropertyValue="${policymgr_https_keystore_credential_alias}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			propertyName=ranger-rms.credential.provider.path
			newPropertyValue="${keystore}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site

			if [ "${keystore}" != "" ]
			then
				propertyName=ranger-rms.service.https.attrib.keystore.pass
				newPropertyValue="_"
				updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
				$PYTHON_COMMAND_INVOKER ranger_credential_helper.py -l "cred/lib/*" -f "$keystore" -k "$policymgr_https_keystore_credential_alias" -v "$ranger_rms_https_keystore_password" -c 1
			else
				propertyName=ranger-rms.service.https.attrib.keystore.pass
				newPropertyValue="${ranger_rms_https_keystore_password}"
				updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
			fi
			if test -f $keystore; then
				chown -R ${unix_user}:${unix_group} ${keystore}
				chmod 640 ${keystore}
			else
				propertyName=ranger-rms.service.https.attrib.keystore.pass
				newPropertyValue="${ranger_rms_https_keystore_password}"
				updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
			fi
		fi
	fi
	##################################
	if [ "${javax_net_ssl_keyStore}" != "" ]  && [ "${javax_net_ssl_keyStorePassword}" != "" ]
	then
		javax_net_ssl_keyStoreAlias=keyStoreAlias

		propertyName=ranger-rms.keystore.file
		newPropertyValue="${javax_net_ssl_keyStore}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default

		propertyName=ranger-rms.keystore.alias
		newPropertyValue="${javax_net_ssl_keyStoreAlias}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default

		if [ "${keystore}" != "" ]
		then
			propertyName=ranger-rms.keystore.password
			newPropertyValue="_"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default

			$PYTHON_COMMAND_INVOKER ranger_credential_helper.py -l "cred/lib/*" -f "$keystore" -k "$javax_net_ssl_keyStoreAlias" -v "$javax_net_ssl_keyStorePassword" -c 1
		else
			propertyName=ranger-rms.keystore.password
			newPropertyValue="${javax_net_ssl_keyStorePassword}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default
		fi

		if test -f $keystore; then
			chown -R ${unix_user}:${unix_group} ${keystore}
		else
			propertyName=ranger-rms.keystore.password
			newPropertyValue="${javax_net_ssl_keyStorePassword}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default
		fi

	fi
	if [ "${javax_net_ssl_trustStore}" != "" ]  && [ "${javax_net_ssl_trustStorePassword}" != "" ]
	then
		javax_net_ssl_trustStoreAlias=trustStoreAlias

		propertyName=ranger-rms.truststore.file
		newPropertyValue="${javax_net_ssl_trustStore}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default

		propertyName=ranger-rms.truststore.alias
		newPropertyValue="${javax_net_ssl_trustStoreAlias}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default
		if [ "${keystore}" != "" ]
		then
			propertyName=ranger-rms.truststore.password
			newPropertyValue="_"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default

			$PYTHON_COMMAND_INVOKER ranger_credential_helper.py -l "cred/lib/*" -f "$keystore" -k "$javax_net_ssl_trustStoreAlias" -v "$javax_net_ssl_trustStorePassword" -c 1
		else
			propertyName=ranger-rms.truststore.password
			newPropertyValue="${javax_net_ssl_trustStorePassword}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default
		fi
		if test -f $keystore; then
			chown -R ${unix_user}:${unix_group} ${keystore}
		else
			propertyName=ranger-rms.truststore.password
			newPropertyValue="${javax_net_ssl_trustStorePassword}"
			updatePropertyToFilePy $propertyName $newPropertyValue $to_file_default
		fi

	fi
	########### HA CONFIG ############
	if [ "${ranger_rms_server_ha_enabled}" == "true" ]
	then
		propertyName=ranger-rms.server.ha.enabled
		newPropertyValue="${ranger_rms_server_ha_enabled}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_zkroot}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.zkroot
		newPropertyValue="${ranger_rms_server_ha_zookeeper_zkroot}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_connect}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.connect
		newPropertyValue="${ranger_rms_server_ha_zookeeper_connect}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ids}" != "" ]
	then
		propertyName=ranger-rms.server.ids
		newPropertyValue="${ranger_rms_server_ids}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_address_id1}" != "" ]
	then
		propertyName=ranger-rms.server.address.id1
		newPropertyValue="${ranger_rms_server_address_id1}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_address_id2}" != "" ]
	then
		propertyName=ranger-rms.server.address.id2
		newPropertyValue="${ranger_rms_server_address_id2}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_address_id3}" != "" ]
	then
		propertyName=ranger-rms.server.address.id3
		newPropertyValue="${ranger_rms_server_address_id3}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_retry_sleeptime_ms}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.retry.sleeptime.ms
		newPropertyValue="${ranger_rms_server_ha_zookeeper_retry_sleeptime_ms}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_session_timeout_ms}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.session.timeout.ms
		newPropertyValue="${ranger_rms_server_ha_zookeeper_session_timeout_ms}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_num_retries}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.num.retries
		newPropertyValue="${ranger_rms_server_ha_zookeeper_num_retries}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_acl}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.acl
		newPropertyValue="${ranger_rms_server_ha_zookeeper_acl}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_server_ha_zookeeper_acl_auth}" != "" ]
	then
		propertyName=ranger-rms.server.ha.zookeeper.acl.auth
		newPropertyValue="${ranger_rms_server_ha_zookeeper_acl_auth}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${hive_metastore_uris}" != "" ]
	then
		propertyName=hive.metastore.uris
		newPropertyValue="${hive_metastore_uris}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${hive_metastore_sasl_enabled}" != "" ]
	then
		propertyName=hive.metastore.sasl.enabled
		newPropertyValue="${hive_metastore_sasl_enabled}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${hive_metastore_client_socket_timeout}" != "" ]
	then
		propertyName=hive.metastore.client.socket.timeout
		newPropertyValue="${hive_metastore_client_socket_timeout}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${hive_metastore_service_host}" != "" ]
	then
		propertyName=hive.metastore.service.host
		newPropertyValue="${hive_metastore_service_host}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_HMS_map_managed_tables}" != "" ]
	then
		propertyName=ranger-rms.HMS.map.managed.tables
		newPropertyValue="${ranger_rms_HMS_map_managed_tables}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_HMS_source_service_name}" != "" ]
	then
		propertyName=ranger-rms.HMS.source.service.name
		newPropertyValue="${ranger_rms_HMS_source_service_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

  if [ "${ranger_rms_HMS_target_service_name}" != "" ]
	then
		propertyName=ranger-rms.HMS.target.service.name
		newPropertyValue="${ranger_rms_HMS_target_service_name}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_polling_notifications_frequency_ms}" != "" ]
	then
		propertyName=ranger-rms.polling.notifications.frequency.ms
		newPropertyValue="${ranger_rms_polling_notifications_frequency_ms}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi

	if [ "${ranger_rms_transaction_size_for_persisting_mappings}" != "" ]
	then
		propertyName=ranger-rms.transaction.size.for.persisting.mappings
		newPropertyValue="${ranger_rms_transaction_size_for_persisting_mappings}"
		updatePropertyToFilePy $propertyName $newPropertyValue $to_file_ranger_rms_site
	fi
}

#=====================================================================

setup_unix_user_group(){
	log "[I] Setting up UNIX user : ${unix_user} and group: ${unix_group}";
	#create group if it does not exist
	egrep "^$unix_group" /etc/group >& /dev/null
	if [ $? -ne 0 ]
	then
		groupadd ${unix_group}
		check_ret_status_for_groupadd $? "Creating group ${unix_group} failed"
	fi

	#create user if it does not exists
	id -u ${unix_user} > /dev/null 2>&1
	if [ $? -ne 0 ]
	then
		check_user_pwd ${unix_user_pwd}
		log "[I] Creating new user and adding to group";
		useradd ${unix_user} -g ${unix_group} -m
		check_ret_status $? "useradd ${unix_user} failed"

		passwdtmpfile=passwd.tmp
		if [  -f "$passwdtmpfile" ]; then
			rm -rf  ${passwdtmpfile}
		fi
		cat> ${passwdtmpfile} << EOF
${unix_user}:${unix_user_pwd}
EOF
		chpasswd <  ${passwdtmpfile}
		rm -rf  ${passwdtmpfile}
	else
		useringroup=`id ${unix_user}`
		useringrouparr=(${useringroup// / })
		if [[  ${useringrouparr[1]} =~ "(${unix_group})" ]]
		then
			log "[I] the ${unix_user} user already exists and belongs to group ${unix_group}"
		else
			log "[I] User already exists, adding it to group ${unix_group}"
			usermod -g ${unix_group} ${unix_user}
		fi
	fi

	log "[I] Setting up UNIX user : ${unix_user} and group: ${unix_group} DONE";
}

setup_install_files(){

	log "[I] Setting up installation files and directory";

	if [ ! -d ${WEBAPP_ROOT}/WEB-INF/classes/conf ]; then
		log "[I] Copying ${WEBAPP_ROOT}/WEB-INF/classes/conf.dist ${WEBAPP_ROOT}/WEB-INF/classes/conf"
		mkdir -p ${WEBAPP_ROOT}/WEB-INF/classes/conf
		cp ${WEBAPP_ROOT}/WEB-INF/classes/conf.dist/* ${WEBAPP_ROOT}/WEB-INF/classes/conf
	fi
	if [ -d ${WEBAPP_ROOT}/WEB-INF/classes/conf ]; then
		chown -R ${unix_user} ${WEBAPP_ROOT}/WEB-INF/classes/conf
		chown -R ${unix_user} ${WEBAPP_ROOT}/WEB-INF/classes/conf/
	fi

	if [ ! -d ${WEBAPP_ROOT}/WEB-INF/classes/lib ]; then
		log "[I] Creating ${WEBAPP_ROOT}/WEB-INF/classes/lib"
		mkdir -p ${WEBAPP_ROOT}/WEB-INF/classes/lib
	fi
	if [ -d ${WEBAPP_ROOT}/WEB-INF/classes/lib ]; then
		chown -R ${unix_user} ${WEBAPP_ROOT}/WEB-INF/classes/lib
	fi

	echo "export RANGER_HADOOP_CONF_DIR=${hadoop_conf}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-hadoopconfdir.sh
		chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-hadoopconfdir.sh

		hadoop_conf_file=${hadoop_conf}/core-site.xml
		ranger_hadoop_conf_file=${WEBAPP_ROOT}/WEB-INF/classes/conf/core-site.xml

		if [ -d ${WEBAPP_ROOT}/WEB-INF/classes/conf ]; then
				chown -R ${unix_user} ${WEBAPP_ROOT}/WEB-INF/classes/conf
				if [ "${hadoop_conf}" == "" ]
				then
						log "[WARN] Property hadoop_conf not found. Creating blank core-site.xml."
						echo "<configuration></configuration>" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/core-site.xml
				else
						if [ -f ${hadoop_conf_file} ]; then
								ln -sf ${hadoop_conf_file} ${WEBAPP_ROOT}/WEB-INF/classes/conf/core-site.xml
						else
								log "[WARN] core-site.xml file not found in provided hadoop_conf path. Creating blank core-site.xml"
								echo "<configuration></configuration>" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/core-site.xml
						fi
				fi
		fi

	if [ -d /etc/init.d ]; then
		log "[I] Setting up init.d"
		cp ${INSTALL_DIR}/${ranger_rms_initd} /etc/init.d/${ranger_rms}
		chmod ug+rx /etc/init.d/${ranger_rms}

		if [ -d /etc/rc2.d ]
		then
			RC_DIR=/etc/rc2.d
			log "[I] Creating script S88${ranger_rms}/K90${ranger_rms} in $RC_DIR directory .... "
			rm -f $RC_DIR/S88${ranger_rms}  $RC_DIR/K90${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/S88${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/K90${ranger_rms}
		fi

		if [ -d /etc/rc3.d ]
		then
			RC_DIR=/etc/rc3.d
			log "[I] Creating script S88${ranger_rms}/K90${ranger_rms} in $RC_DIR directory .... "
			rm -f $RC_DIR/S88${ranger_rms}  $RC_DIR/K90${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/S88${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/K90${ranger_rms}
		fi

		# SUSE has rc2.d and rc3.d under /etc/rc.d
		if [ -d /etc/rc.d/rc2.d ]
		then
			RC_DIR=/etc/rc.d/rc2.d
			log "[I] Creating script S88${ranger_rms}/K90${ranger_rms} in $RC_DIR directory .... "
			rm -f $RC_DIR/S88${ranger_rms}  $RC_DIR/K90${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/S88${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/K90${ranger_rms}
		fi
		if [ -d /etc/rc.d/rc3.d ]
		then
			RC_DIR=/etc/rc.d/rc3.d
			log "[I] Creating script S88${ranger_rms}/K90${ranger_rms} in $RC_DIR directory .... "
			rm -f $RC_DIR/S88${ranger_rms}  $RC_DIR/K90${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/S88${ranger_rms}
			ln -s /etc/init.d/${ranger_rms} $RC_DIR/K90${ranger_rms}
		fi
	fi

	if [  -f /etc/init.d/${ranger_rms} ]; then
		if [ "${unix_user}" != "" ]; then
			sed  's/^LINUX_USER=.*$/LINUX_USER='${unix_user}'/g' -i  /etc/init.d/${ranger_rms}
		fi
	fi

	if [ -z "${RMS_LOG_DIR}" ] || [ ${RMS_LOG_DIR} == ${RMS_DIR} ]; then
			RMS_LOG_DIR=${RMS_DIR}/ews/logs;
	fi

	if [ ! -d ${RMS_LOG_DIR} ]; then
		log "[I] ${RMS_LOG_DIR} RMS Log folder"
		mkdir -p ${RMS_LOG_DIR}
	fi

	if [ -d ${RMS_LOG_DIR} ]; then
		chown -R ${unix_user} ${RMS_LOG_DIR}
	fi

	echo "export RMS_LOG_DIR=${RMS_LOG_DIR}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-logdir.sh
	chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-logdir.sh

	if [ -z "${RMS_PID_DIR_PATH}" ]
	then
		RMS_PID_DIR_PATH=/var/run/ranger-rms
	fi

	if [ ! -d ${RMS_PID_DIR_PATH} ]; then
		log "[I] Creating ranger-rms PID folder: ${RMS_PID_DIR_PATH}"
		mkdir -p ${RMS_PID_DIR_PATH}
		if [ ! $? = "0" ];then
			log "Make $RMS_PID_DIR_PATH failure....!!";
			exit 1;
		fi
	fi

	chown -R ${unix_user} ${RMS_PID_DIR_PATH}

	echo "export RMS_PID_DIR_PATH=${RMS_PID_DIR_PATH}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-piddir.sh
	echo "export RMS_USER=${unix_user}" >> ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-piddir.sh
	chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-piddir.sh

	if [ "${db_ssl_verifyServerCertificate}" == "true" ]
	then
		if [ "${db_ssl_auth_type}" == "1-way" ]
		then
			DB_SSL_PARAM="' -Djavax.net.ssl.trustStore=${javax_net_ssl_trustStore} -Djavax.net.ssl.trustStorePassword=${javax_net_ssl_trustStorePassword} '"
		else
			DB_SSL_PARAM="' -Djavax.net.ssl.keyStore=${javax_net_ssl_keyStore} -Djavax.net.ssl.keyStorePassword=${javax_net_ssl_keyStorePassword} -Djavax.net.ssl.trustStore=${javax_net_ssl_trustStore} -Djavax.net.ssl.trustStorePassword=${javax_net_ssl_trustStorePassword} '"
		fi
		echo "export DB_SSL_PARAM=${DB_SSL_PARAM}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-dbsslparam.sh
		chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-dbsslparam.sh
	else
		if [ -f ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-dbsslparam.sh ]; then
			DB_SSL_PARAM=""
			echo "export DB_SSL_PARAM=${DB_SSL_PARAM}" > ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-dbsslparam.sh
			chmod a+rx ${WEBAPP_ROOT}/WEB-INF/classes/conf/ranger-rms-env-dbsslparam.sh
		fi
	fi

	log "[I] Setting up installation files and directory DONE";

	if [ ! -f ${INSTALL_DIR}/rpm ]; then
		if [ -d ${INSTALL_DIR} ]
		then
			chown -R ${unix_user}:${unix_group} ${INSTALL_DIR}
			chown -R ${unix_user}:${unix_group} ${INSTALL_DIR}/*
		fi
	fi

	# Copy ranger-rms-services to /usr/bin
	if [ ! \( -e /usr/bin/ranger-rms \) ]
	then
		ln -sf ${INSTALL_DIR}/ranger-rms /usr/bin/ranger-rms
		chmod ug+rx /usr/bin/ranger-rms
	fi

	if [ ! \( -e /usr/bin/ranger-rms-services.sh \) ]
	then
		ln -sf ${INSTALL_DIR}/ranger-rms /usr/bin/ranger-rms-services.sh
		chmod ug+rx /usr/bin/ranger-rms-services.sh
	fi

	if [ ! \( -e ${INSTALL_DIR}/ranger-rms-services.sh \) ]
	then
		ln -sf ${INSTALL_DIR}/ranger-rms-initd ${INSTALL_DIR}/ranger-rms-services.sh
		chmod ug+rx ${INSTALL_DIR}/ranger-rms-services.sh
	fi
	if [ ! -d /var/log/ranger/ranger-rms ]; then
		mkdir -p /var/log/ranger/ranger-rms
		if [ -d ews/logs ]; then
			cp -r ews/logs/* /var/log/ranger/ranger-rms
		fi
	fi
	if [ -d /var/log/ranger/ranger-rms ]; then
		chmod 755 /var/log/ranger/ranger-rms
		chown -R $unix_user:$unix_group /var/log/ranger/ranger-rms
	fi
}

log " --------- Running RMS Application Install Script --------- "
log "[I] uname=`uname`"
log "[I] hostname=`hostname`"
init_variables
get_distro
check_java_version
check_db_connector
setup_unix_user_group
setup_install_files
sanity_check_files
copy_db_connector
check_python_command

#run_dba_steps
#if [ "$?" == "0" ]
#then
#	$PYTHON_COMMAND_INVOKER db_setup.py
#else
#	exit 1
#fi
#if [ "$?" == "0" ]
#then
#	update_properties
#	$PYTHON_COMMAND_INVOKER db_setup.py -javapatch
#else
#	log "[E] DB schema setup failed! Please contact Administrator."
#	exit 1
#fi

update_properties

echo "Installation of RMS is completed."
