/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.ranger.rms.ha;

import org.apache.hadoop.conf.Configuration;
import org.apache.ranger.rms.biz.MetadataSyncScheduler;
import org.apache.ranger.rms.ha.service.HARmsService;
import org.apache.ranger.rms.server.RMSConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class MetadataSyncServiceHandler implements HARmsService, ActiveStateChangeHandler {

	private static final Logger LOG = LoggerFactory.getLogger(MetadataSyncServiceHandler.class);

	private final Configuration configuration;

	@Autowired
	public MetadataSyncServiceHandler() {
		this.configuration = RMSConfig.getInstance();
	}

	@Override
	public void instanceIsActive() throws Exception {
		LOG.info("==> MetadataSyncServiceHandler.instanceIsActive()");
		startInternal();
	}

	@Override
	public void instanceIsPassive() throws Exception {
		stop();
	}

	@Override
	public void start() throws Exception {
		LOG.info("==> MetadataSyncServiceHandler.start()");

		if (!HAConfiguration.isHAEnabled(configuration)) {
			startInternal();
		} else {
			LOG.info("MetadataSyncServiceHandler.start(): deferring service start until instance activation");
		}
	}

	@Override
	public void stop() throws Exception {
		MetadataSyncScheduler.metaDataSyncSchedulerFlag = false;
		LOG.info("MetadataSyncServiceHandler.stop(): stopped with metaDataSyncSchedulerFlag: "
				+ MetadataSyncScheduler.metaDataSyncSchedulerFlag);

	}

	void startInternal() {
		try {
			MetadataSyncScheduler.metaDataSyncSchedulerFlag = true;
		} catch (Exception ex) {
			LOG.error("MetadataSyncServiceHandler: failed to change metaDataSyncFlag", ex);
		}
	}
}
