/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ranger.patch;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.ranger.biz.ServiceDBStore;
import org.apache.ranger.common.GUIDUtil;
import org.apache.ranger.common.JSONUtil;
import org.apache.ranger.common.RangerValidatorFactory;
import org.apache.ranger.db.RangerDaoManager;
import org.apache.ranger.entity.XXService;
import org.apache.ranger.entity.XXServiceDef;
import org.apache.ranger.plugin.model.RangerPolicy;
import org.apache.ranger.plugin.model.RangerPolicyResourceSignature;
import org.apache.ranger.plugin.model.RangerServiceDef;
import org.apache.ranger.plugin.model.validation.RangerServiceDefValidator;
import org.apache.ranger.plugin.model.validation.RangerValidator.Action;
import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;
import org.apache.ranger.rest.ServiceRESTUtil;
import org.apache.ranger.util.CLIUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class PatchForHiveServiceDefUpdate_J10048 extends BaseLoader {
	private static final Logger logger = Logger.getLogger(PatchForHiveServiceDefUpdate_J10048.class);
	public static final String SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME  = "hive";
	public static final String WILDCARD_ASTERISK  				          = "*";
	public static final String RESOURCE_STORAGE_TYPE 			          = "storage-type";
	public static final String RESOURCE_STORAGE_URL    			          = "storage-url";
	public static final String LOGIN_ID_ADMIN 					          = "admin";
	public static final String POlICY_NAME_FOR_ALL_STORAGE_TYPE		      = "all - storage";
	public static final String ACCESS_TYPE_RWSTORAGE	   				  = "rwstorage";


	@Autowired
	RangerDaoManager daoMgr;

	@Autowired
	JSONUtil jsonUtil;

	@Autowired
	GUIDUtil guidUtil;

	@Autowired
	RangerValidatorFactory validatorFactory;

	@Autowired
	ServiceDBStore svcDBStore;

	public static void main(String[] args) {
		logger.info("main()");
		try {
			PatchForHiveServiceDefUpdate_J10048 loader = (PatchForHiveServiceDefUpdate_J10048) CLIUtil.getBean(PatchForHiveServiceDefUpdate_J10048.class);
			loader.init();
			while (loader.isMoreToProcess()) {
				loader.load();
			}
			logger.info("Load complete. Exiting!!!");
			System.exit(0);
		} catch (Exception e) {
			logger.error("Error loading", e);
			System.exit(1);
		}
	}

	@Override
	public void init() throws Exception {
		// Do Nothing
	}

	@Override
	public void execLoad() {
		logger.info("==> PatchForHiveServiceDefUpdate_J10048.execLoad()");
		try {
			updateHiveServiceDef();
		} catch (Exception e) {
			logger.error("Error whille updateHiveServiceDef()data.", e);
		}
		logger.info("<== PatchForHiveServiceDefUpdate_J10048.execLoad()");
	}

	@Override
	public void printStats() {
		logger.info("PatchForHiveServiceDefUpdateForStorageTypeResource data ");
	}

	private void updateHiveServiceDef() {
		RangerServiceDef ret = null;
		RangerServiceDef embeddedHiveServiceDef = null;
		RangerServiceDef dbHiveServiceDef = null;
		List<RangerServiceDef.RangerResourceDef> embeddedHiveResourceDefs = null;
		List<RangerServiceDef.RangerAccessTypeDef> embeddedHiveAccessTypes = null;
		XXServiceDef xXServiceDefObj = null;
		try {
			embeddedHiveServiceDef = EmbeddedServiceDefsUtil.instance().getEmbeddedServiceDef(SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME);
			if (embeddedHiveServiceDef != null) {

				xXServiceDefObj = daoMgr.getXXServiceDef().findByName(SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME);
				Map<String, String> serviceDefOptionsPreUpdate = null;
				String jsonStrPreUpdate = null;
				if (xXServiceDefObj != null) {
					jsonStrPreUpdate = xXServiceDefObj.getDefOptions();
					serviceDefOptionsPreUpdate = jsonStringToMap(jsonStrPreUpdate);
					xXServiceDefObj = null;
				}
				dbHiveServiceDef = svcDBStore.getServiceDefByName(SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME);

				if (dbHiveServiceDef != null) {
					embeddedHiveResourceDefs = embeddedHiveServiceDef.getResources();
					embeddedHiveAccessTypes = embeddedHiveServiceDef.getAccessTypes();

					if (checkStorageTyperesourcePresent(embeddedHiveResourceDefs)) {
						// This is to check if STORAGE-TYPE resource is added to the resource definition, if so update the resource def and accessType def
						if (embeddedHiveResourceDefs != null) {
							dbHiveServiceDef.setResources(embeddedHiveResourceDefs);
						}
						if (embeddedHiveAccessTypes != null) {
							if (!embeddedHiveAccessTypes.toString().equalsIgnoreCase(dbHiveServiceDef.getAccessTypes().toString())) {
								dbHiveServiceDef.setAccessTypes(embeddedHiveAccessTypes);
							}
						}
					}

					RangerServiceDefValidator validator = validatorFactory.getServiceDefValidator(svcDBStore);
					validator.validate(dbHiveServiceDef, Action.UPDATE);

					ret = svcDBStore.updateServiceDef(dbHiveServiceDef);
					if (ret == null) {
						logger.error("Error while updating " + SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME + "service-def");
						throw new RuntimeException("Error while updating " + SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME + "service-def");
					}
					xXServiceDefObj = daoMgr.getXXServiceDef().findByName(SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME);
					if (xXServiceDefObj != null) {
						String jsonStrPostUpdate = xXServiceDefObj.getDefOptions();
						Map<String, String> serviceDefOptionsPostUpdate = jsonStringToMap(jsonStrPostUpdate);
						if (serviceDefOptionsPostUpdate != null && serviceDefOptionsPostUpdate.containsKey(RangerServiceDef.OPTION_ENABLE_DENY_AND_EXCEPTIONS_IN_POLICIES)) {
							if (serviceDefOptionsPreUpdate == null || !serviceDefOptionsPreUpdate.containsKey(RangerServiceDef.OPTION_ENABLE_DENY_AND_EXCEPTIONS_IN_POLICIES)) {
								String preUpdateValue = serviceDefOptionsPreUpdate == null ? null : serviceDefOptionsPreUpdate.get(RangerServiceDef.OPTION_ENABLE_DENY_AND_EXCEPTIONS_IN_POLICIES);
								if (preUpdateValue == null) {
									serviceDefOptionsPostUpdate.remove(RangerServiceDef.OPTION_ENABLE_DENY_AND_EXCEPTIONS_IN_POLICIES);
								} else {
									serviceDefOptionsPostUpdate.put(RangerServiceDef.OPTION_ENABLE_DENY_AND_EXCEPTIONS_IN_POLICIES, preUpdateValue);
								}
								xXServiceDefObj.setDefOptions(mapToJsonString(serviceDefOptionsPostUpdate));
								daoMgr.getXXServiceDef().update(xXServiceDefObj);
							}
						}
						createDefaultPolicyForStoreType();
					}
				}
			}
		} catch (Exception e) {
			logger.error("Error while updating " + SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME + "service-def", e);
		}
	}

	private boolean checkStorageTyperesourcePresent(List<RangerServiceDef.RangerResourceDef> resourceDefs) {
		boolean ret = false;
		for(RangerServiceDef.RangerResourceDef resourceDef : resourceDefs) {
			if (RESOURCE_STORAGE_TYPE.equals(resourceDef.getName()) ) {
				ret = true ;
				break;
			}
		}
		return ret;
	}

	private String mapToJsonString(Map<String, String> map) throws Exception {
		String ret = null;
		if(map != null) {
			ret = jsonUtil.readMapToString(map);
		}
		return ret;
	}

	protected Map<String, String> jsonStringToMap(String jsonStr) {
		Map<String, String> ret = null;
		if(!StringUtils.isEmpty(jsonStr)) {
			try {
				ret = jsonUtil.jsonToMap(jsonStr);
			} catch(Exception excp) {
				// fallback to earlier format: "name1=value1;name2=value2"
				for(String optionString : jsonStr.split(";")) {
					if(StringUtils.isEmpty(optionString)) {
						continue;
					}
					String[] nvArr = optionString.split("=");
					String name  = (nvArr != null && nvArr.length > 0) ? nvArr[0].trim() : null;
					String value = (nvArr != null && nvArr.length > 1) ? nvArr[1].trim() : null;
					if(StringUtils.isEmpty(name)) {
						continue;
					}
					if(ret == null) {
						ret = new HashMap<String, String>();
					}
					ret.put(name, value);
				}
			}
		}
		return ret;
	}

	private void createDefaultPolicyForStoreType() throws Exception{
		RangerServiceDef rangerServiceDef = svcDBStore.getServiceDefByName(SERVICEDBSTORE_SERVICEDEFBYNAME_HIVE_NAME);
		List<XXService> dbServices = daoMgr.getXXService().findByServiceDefId(rangerServiceDef.getId());
		if (CollectionUtils.isNotEmpty(dbServices)) {
			for (XXService dbService : dbServices) {
				addDefaultPolicyStorageType(dbService.getName(), null);
			}
		}
	}

	private void addDefaultPolicyStorageType(String serviceName, String zoneName) throws Exception {
		logger.info("===> addDefaultPolicyStorageType ServiceName : " + serviceName + " ZoneName : " + zoneName);

		RangerPolicy storageTypePolicy = addPolicyStorageType(serviceName, zoneName);
		List<RangerPolicy> policies = svcDBStore.getPoliciesByResourceSignature(serviceName, storageTypePolicy.getResourceSignature(), true);

		if (CollectionUtils.isNotEmpty(policies)) {
			for (RangerPolicy rangerPolicy : policies) {
				if (logger.isDebugEnabled()) {
					logger.debug("print Policy: " + rangerPolicy);
					logger.debug("policy found with resource " + rangerPolicy.getResources()
							+ " and ResourceSignature " + rangerPolicy.getResourceSignature()
							+ " service name : " + rangerPolicy.getService());
				}
				ServiceRESTUtil.processApplyPolicy(rangerPolicy, storageTypePolicy);
				logger.info("policy after merge : "+rangerPolicy);
				svcDBStore.updatePolicy(rangerPolicy);
			}
		} else {
			logger.info("No policy found with resource storageType = * and storageUrl = * creating new policy");
			svcDBStore.createPolicy(storageTypePolicy);
		}
		logger.info("<=== addDefaultPolicyStorageType");
	}


	private RangerPolicy addPolicyStorageType( String serviceName, String zoneName ) {
		logger.info("===> addPolicyStorageType ");
		RangerPolicy policy;
		Map<String, RangerPolicy.RangerPolicyResource> policyResources = new HashMap<>();
		policyResources.put(RESOURCE_STORAGE_TYPE, new RangerPolicy.RangerPolicyResource(WILDCARD_ASTERISK));
		policyResources.put(RESOURCE_STORAGE_URL, new RangerPolicy.RangerPolicyResource(WILDCARD_ASTERISK));
		policy = new RangerPolicy();
		policy.setService(serviceName);
		policy.setName(POlICY_NAME_FOR_ALL_STORAGE_TYPE);
		policy.setDescription(POlICY_NAME_FOR_ALL_STORAGE_TYPE);
		policy.setIsAuditEnabled(true);
		policy.setCreatedBy(LOGIN_ID_ADMIN);
		policy.setResources(policyResources);
		policy.setPolicyType(RangerPolicy.POLICY_TYPE_ACCESS);
		policy.setGuid(guidUtil.genGUID());
		policy.setZoneName(zoneName);
		RangerPolicy.RangerPolicyItem policyItem= getStorateTypePolicyItem();
		policy.getPolicyItems().add(policyItem);
		policy.setResourceSignature(new RangerPolicyResourceSignature(policy).getSignature() );
		if (logger.isDebugEnabled()) {
			logger.debug("===> getPolicyForAllStorageType policy ResourceSignature  " + policy.getResourceSignature());
			logger.debug("===> getPolicyForAllStorageType policy : " + policy);
		}
		logger.info("<=== addPolicyStorageType ");
		return policy;
	}

	private RangerPolicy.RangerPolicyItem getStorateTypePolicyItem() {
		RangerPolicy.RangerPolicyItem policyItem = new RangerPolicy.RangerPolicyItem();
		policyItem.setUsers(Collections.singletonList("hive"));
		policyItem.setAccesses(Collections.singletonList(new RangerPolicy.RangerPolicyItemAccess(ACCESS_TYPE_RWSTORAGE)));
		policyItem.setDelegateAdmin(false);
		return policyItem;
	}
}

