/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ranger.patch;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.ranger.biz.ServiceDBStore;
import org.apache.ranger.biz.XUserMgr;
import org.apache.ranger.common.GUIDUtil;
import org.apache.ranger.db.RangerDaoManager;
import org.apache.ranger.entity.XXService;
import org.apache.ranger.entity.XXServiceConfigMap;
import org.apache.ranger.entity.XXUser;
import org.apache.ranger.plugin.model.RangerPolicy;
import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItem;
import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyItemAccess;
import org.apache.ranger.plugin.model.RangerPolicy.RangerPolicyResource;
import org.apache.ranger.plugin.model.RangerPolicyResourceSignature;
import org.apache.ranger.plugin.model.RangerServiceDef;
import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;
import org.apache.ranger.rest.ServiceRESTUtil;
import org.apache.ranger.util.CLIUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class PatchForHiveServiceUpdate_J10042 extends BaseLoader {

	private static final Logger logger = Logger.getLogger(PatchForHiveServiceUpdate_J10042.class);
	public static final String RANGERRMS_USER_NAME 				= "rangerrms";
	public static final String ACCESS_TYPE_SELECT 				= "select";
	public static final String ALLOWED_USER_LIST_FOR_TAG_DOWNLOAD = "tag.download.auth.users";
	public static final String POLICY_DOWNLOAD_USERS 			= "policy.download.auth.users";
	public static final String WILDCARD_ASTERISK  				= "*";
	public static final String RESOURCE_DATABASE  				= "database";
	public static final String RESOURCE_TABLE     				= "table";
	public static final String HDFS_SERVICE_USER     			= "hdfs";
	private static final String LOGIN_ID_ADMIN 					= "admin";
	private static final String POlICY_NAME_FOR_ALL_TABLE 		= "all - database, table";

	@Autowired
	RangerDaoManager daoMgr;

	@Autowired
	ServiceDBStore svcDBStore;

	@Autowired
	GUIDUtil guidUtil;

	@Autowired
	XUserMgr xUserMgr;

	@Override
	public void init() throws Exception {
		// Do Nothing
	}

	public static void main(String[] args) {
		logger.info("main()");
		try {
			PatchForHiveServiceUpdate_J10042 loader = (PatchForHiveServiceUpdate_J10042) CLIUtil.getBean(PatchForHiveServiceUpdate_J10042.class);
			loader.init();
			while (loader.isMoreToProcess()) {
				loader.load();
			}
			logger.info("Load complete. Exiting.");
			System.exit(0);
		} catch (Exception e) {
			logger.error("Error loading", e);
			System.exit(1);
		}
	}

	@Override
	public void printStats() {
		logger.info("PatchForHiveServiceUpdate_J10042 ");
	}

	@Override
	public void execLoad() {
		logger.info("==> PatchForHiveServiceUpdate_J10042.execLoad()");
		try {
			if (!updateHiveServiceConfigAndPolicy()) {
				logger.error("Failed to apply the patch.");
				System.exit(1);
			}
		} catch (Exception e) {
			logger.error("Error while applying PatchForHiveServiceUpdate_J10042...", e);
		}
		logger.info("<== PatchForHiveServiceUpdate_J10042.execLoad()");
	}

	private boolean updateHiveServiceConfigAndPolicy() throws Exception {
		logger.info("==> updateHiveServiceConfigAndPolicy ");
		RangerServiceDef embeddedHiveServiceDef;

		embeddedHiveServiceDef = EmbeddedServiceDefsUtil.instance()
				.getEmbeddedServiceDef(EmbeddedServiceDefsUtil.EMBEDDED_SERVICEDEF_HIVE_NAME);

		if (embeddedHiveServiceDef != null) {
			List<XXService> dbServices = daoMgr.getXXService().findByServiceDefId(embeddedHiveServiceDef.getId());
			if (CollectionUtils.isNotEmpty(dbServices)) {
				XXUser xxUser = daoMgr.getXXUser().findByUserName(RANGERRMS_USER_NAME);
				if (null == xxUser) {
					logger.info("rangerrms user is not found, adding new rangerrms user ");
					xUserMgr.createServiceConfigUser(RANGERRMS_USER_NAME);
				} else {
					logger.info("rangerrms user is found.");
				}
				for (XXService dbService : dbServices) {
					logger.info("===> porcessing start for service : "+dbService.getName() );
					updateHiveServiceConfig(dbService.getId(),ALLOWED_USER_LIST_FOR_TAG_DOWNLOAD);
					updateHiveServiceConfig(dbService.getId(),POLICY_DOWNLOAD_USERS);
					addPolicyItemForRangerrmsUser(dbService.getName() , null);
					List<String> securityZoneNames = daoMgr.getXXSecurityZoneDao().findZonesByServiceName(dbService.getName());
					if(null != securityZoneNames ) {
						for(String securityZoneName : securityZoneNames) {
							logger.info("Calling addPolicyItemForRangerrmsUser for service : "+dbService.getName() +" and ZoneName : "+ securityZoneName);
							addPolicyItemForRangerrmsUser(dbService.getName(), securityZoneName);
						}
					}else {
						logger.info("No zone exist for service : "+dbService.getName());
					}
					logger.info("<=== porcessing completed for service : "+dbService.getName() );
				}
			}else {
				logger.info("Hive service does not exist.");
			}
		} else {
			logger.error("The embedded Hive service-definition does not exist.");
			return false;
		}
		logger.info("<== updateHiveServiceConfigAndPolicy ");
		return true;
	}

	private void updateHiveServiceConfig(Long serviceId, String configKey) {
		logger.info("==> updateHiveServiceConfig serviceId: "+serviceId + " configKey : "+configKey);
		List<String> serviceUserList;
		XXServiceConfigMap serviceConfigMap = daoMgr.getXXServiceConfigMap().findByServiceAndConfigKey(serviceId, configKey);
		if (null != serviceConfigMap) {
			String configValue = serviceConfigMap.getConfigvalue();
			serviceUserList = getServiceUserList(configValue);
			if (!serviceUserList.contains(HDFS_SERVICE_USER)) {
				if(logger.isDebugEnabled()) {
				logger.debug("==> updateHiveServiceConfig configValue : " + configValue + " for configkey "
						+ serviceConfigMap.getConfigkey() + " for serviceId : "+serviceId);
				}
				serviceUserList.add(HDFS_SERVICE_USER);
				serviceConfigMap.setConfigvalue(StringUtils.join(serviceUserList, ","));
				daoMgr.getXXServiceConfigMap().update(serviceConfigMap);
			}
		}else {
			logger.info("No service config map found for key : "+configKey + " with serviceId : "+serviceId);
		}
		logger.info("<== updateHiveServiceConfig ");
	}

	private List<String> getServiceUserList( String serviceUsers ) {
		ArrayList<String> serviceUserList = new ArrayList<>();
		if (!StringUtils.isEmpty(serviceUsers)) {
			String[] userList = serviceUsers.split(",");
			for (String userName : userList) {
				if (!StringUtils.isEmpty(userName)) {
					serviceUserList.add(userName.trim());
				}
			}
		}
		return serviceUserList;
	}

	private RangerPolicyItem getRMSPolicyItem() {
		RangerPolicyItem policyItemRangerRms = new RangerPolicyItem();
		policyItemRangerRms.setUsers(Collections.singletonList(RANGERRMS_USER_NAME));
		policyItemRangerRms.setAccesses(Collections.singletonList(new RangerPolicyItemAccess(ACCESS_TYPE_SELECT)));
		policyItemRangerRms.setDelegateAdmin(false);
		return policyItemRangerRms;
	}

	private RangerPolicy getPolicyForAllTables( String serviceName, String zoneName ) {
		logger.info("===> getPolicyForAllTables ");
		RangerPolicy policy;
		Map<String, RangerPolicyResource> policyResources = new HashMap<>();
		policyResources.put(RESOURCE_DATABASE, new RangerPolicyResource(WILDCARD_ASTERISK));
		policyResources.put(RESOURCE_TABLE, new RangerPolicyResource(WILDCARD_ASTERISK));
		policy = new RangerPolicy();
		policy.setService(serviceName);
		policy.setName("grant-rangerrms-" + System.currentTimeMillis());
		policy.setDescription(POlICY_NAME_FOR_ALL_TABLE);
		policy.setIsAuditEnabled(true);
		policy.setCreatedBy(LOGIN_ID_ADMIN);
		policy.setResources(policyResources);
		policy.setPolicyType(RangerPolicy.POLICY_TYPE_ACCESS);
		policy.setGuid(guidUtil.genGUID());
		policy.setZoneName(zoneName);
		RangerPolicyItem policyItemRangerRms = getRMSPolicyItem();
		policy.getPolicyItems().add(policyItemRangerRms);
		policy.setResourceSignature(new RangerPolicyResourceSignature(policy).getSignature() );
		if (logger.isDebugEnabled()) {
			logger.debug("===> getPolicyForAllTables policy ResourceSignature  " + policy.getResourceSignature());
			logger.debug("===> getPolicyForAllTables policy : " + policy);
		}
		logger.info("<=== getPolicyForAllTables ");
		return policy;
	}

	private void addPolicyItemForRangerrmsUser(String serviceName, String zoneName) throws Exception {
		logger.info("===> addPolicyItemForRangerrmsUser serviceName : " + serviceName + " zoneName : " + zoneName);

		RangerPolicy allTablePolicy = getPolicyForAllTables(serviceName, zoneName);
		List<RangerPolicy> policies = svcDBStore.getPoliciesByResourceSignature(serviceName, allTablePolicy.getResourceSignature(), true);

		if (CollectionUtils.isNotEmpty(policies)) {
			for (RangerPolicy rangerPolicy : policies) {
				if (logger.isDebugEnabled()) {
					logger.debug("print Policy: " + rangerPolicy);
					logger.debug("policy found with resource " + rangerPolicy.getResources()
							+ " and ResourceSignature " + rangerPolicy.getResourceSignature()
							+ " service name : " + rangerPolicy.getService());
				}
				ServiceRESTUtil.processApplyPolicy(rangerPolicy, allTablePolicy);
				logger.info("policy after merge : "+rangerPolicy);
				svcDBStore.updatePolicy(rangerPolicy);
			}
		} else {
				logger.info("No policy found with resource database=* and table=* creating new policy");
				svcDBStore.createPolicy(allTablePolicy);
		}
		logger.info("<=== addPolicyItemForRangerrmsUser ");
	}
}